@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.13 98/03/06 07:22:47 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.32 2000/03/16 15:20:58 abe Exp $";
 #endif
 
 
@@ -40,7 +40,9 @@ static char *rcsid = "$Id: arg.c,v 1.13 98/03/06 07:22:47 abe Exp $";
 
 
 _PROTOTYPE(static int ckfd_range,(char *first, char *dash, char *last, int *lo, int *hi));
+_PROTOTYPE(static void enter_fd_lst,(char *nm, int lo, int hi));
 _PROTOTYPE(static int enter_nwad,(struct nwad *n, int sp, int ep, char *s, struct hostent *he));
+_PROTOTYPE(static struct hostent *lkup_hostnm,(char *hn, struct nwad *n));
 
 
 /*
@@ -97,14 +99,18 @@ FD_range_nondigit:
  */
 
 int
-ck_file_arg(i, ac, av)
+ck_file_arg(i, ac, av, fv, rs, sbp)
 	int i;			/* first file argument index */
 	int ac;			/* argument count */
 	char *av[];		/* argument vector */
+	int fv;			/* Ffilesys value (real or temporary) */
+	int rs;			/* Readlink() status if argument count == 1:
+				 *	0 = undone; 1 = done */
+	struct stat *sbp;	/* if non-NULL, pointer to stat(2) buffer
+				 * when argument count == 1 */
 {
 	unsigned char ad, an;
 	short err = 0;
-	int errno_save;
 	char *fnm, *fsnm, *path;
 	int fsm, ftype, j;
 	MALLOC_S l;
@@ -114,13 +120,14 @@ ck_file_arg(i, ac, av)
 	static int nma = 0;
 	struct stat sb;
 	struct sfile *sfp;
+	short ss = 0;
 
 #if	defined(CKFA_EXPDEV)
 	dev_t dev, rdev;
 #endif	/* defined(CKFA_EXPDEV) */
 
 #if	defined(HASPROCFS)
-	short pfsnl = -1;
+	int pfsnl = -1;
 	pid_t pid;
 	struct procfsid *pfi;
 #endif	/* defined(HASPROCFS) */
@@ -129,9 +136,13 @@ ck_file_arg(i, ac, av)
  * Loop through arguments.
  */
 	for (; i < ac; i++) {
-	    if (!(path = Readlink(av[i]))) {
-		err = 1;
-		continue;
+	    if (rs && (ac == 1) && (i == 0))
+		path = av[i];
+	    else {
+		if (!(path = Readlink(av[i]))) {
+		    ErrStat = 1;
+		    continue;
+		}
 	    }
 	/*
 	 * Remove a terminating `/' from Readlink()-expanded path, but never
@@ -142,14 +153,14 @@ ck_file_arg(i, ac, av)
 	 */
 	    if (path != av[i] && (j = strlen(path)) > 1 && path[j-1] == '/')
 		path[j-1] = '\0';
-	    for (ftype = 1, mp = Mtab, nm = 0;
-		 (Ffilesys != 1) && mp;
+	    for (ftype = 1, mp = readmnt(), nm = 0;
+		 (fv != 1) && mp;
 		 mp = mp->next)
 	    {
 		fsm = 0;
 		if (strcmp(mp->dir, path) == 0)
 		    fsm++;
-		else if (Ffilesys == 2 || (mp->fs_mode & S_IFMT) == S_IFBLK) {
+		else if (fv == 2 || (mp->fs_mode & S_IFMT) == S_IFBLK) {
 		    if (mp->fsnmres && strcmp(mp->fsnmres, path) == 0)
 			fsm++;
 		}
@@ -187,10 +198,10 @@ ck_file_arg(i, ac, av)
 		}
 		mmp[nm++] = mp;
 	    }
-	    if (Ffilesys == 2 && nm == 0) {
+	    if (fv == 2 && nm == 0) {
 		(void) fprintf(stderr, "%s: not a file system: ", Pn);
 		safestrprt(av[i], stderr, 1);
-		err = 1;
+		ErrStat = 1;
 		continue;
 	    }
 	/*
@@ -221,13 +232,20 @@ ck_file_arg(i, ac, av)
 		/*
 		 * Stat the path to obtain its characteristics.
 		 */
-		    if (statsafely(fnm, &sb) != 0) {
-			errno_save = errno;
-			(void) fprintf(stderr, "%s: status error on ", Pn);
-			safestrprt(fnm, stderr, 0);
-			(void) fprintf(stderr, ": %s\n", strerror(errno_save));
-			err = 1;
-			break;
+		    if (sbp && (ac == 1))
+			sb = *sbp;
+		    else {
+			if (statsafely(fnm, &sb) != 0) {
+			    int en = errno;
+
+			    (void) fprintf(stderr, "%s: status error on ", Pn);
+			    safestrprt(fnm, stderr, 0);
+			    (void) fprintf(stderr, ": %s\n", strerror(en));
+			    Sfile = sfp->next;
+			    (void) free((FREE_P *)sfp);
+			    ErrStat = 1;
+			    continue;
+			}
 		    }
 		    sfp->i = sb.st_ino;
 		    sfp->mode = sb.st_mode & S_IFMT;
@@ -235,8 +253,8 @@ ck_file_arg(i, ac, av)
 #if	defined(CKFA_EXPDEV)
 		/*
 		 * Expand device numbers before saving, so that they match the
-		 * already-expanded Mtab device numbers.  (This is an EP/IX
-		 * 2.1.1 and above artifact.)
+		 * already-expanded local mount info table device numbers.
+		 * (This is an EP/IX 2.1.1 and above artifact.)
 		 */
 		    dev = expdev(sb.st_dev);
 		    rdev = expdev(sb.st_rdev);
@@ -270,6 +288,7 @@ ck_file_arg(i, ac, av)
 
 		} else {
 		    mp = mmp[mx++];
+		    ss++;
 
 #if	defined(HASPROCFS)
 		/*
@@ -295,6 +314,7 @@ ck_file_arg(i, ac, av)
 		    sfp->i = mp->inode;
 		    sfp->mode = mp->mode & S_IFMT;
 		}
+		ss = 1;		/* indicate a "safe" stat() */
 	    /*
 	     * Store the file name and file system name pointers in the sfile
 	     * structure, allocating space as necessary.
@@ -337,10 +357,10 @@ ck_file_arg(i, ac, av)
 		if (!Mtprocfs || Procsrch)
 		    continue;
 
-# if	defined(HASFSTYPE)
+# if	defined(HASFSTYPE) && HASFSTYPE==1
 		if (strcmp(sb.st_fstype, HASPROCFS) != 0)
 		    continue;
-# endif	/* defined(HASFSTYPE) */
+# endif	/* defined(HASFSTYPE) && HASFSTYPE==1 */
 
 		if (pfsnl == -1)
 		    pfsnl = strlen(Mtprocfs->dir);
@@ -358,7 +378,7 @@ ck_file_arg(i, ac, av)
 
 		else {
 		    for (j = pfsnl+1; path[j]; j++) {
-			if (!isdigit(path[j]))
+			if (!isdigit((unsigned char)path[j]))
 			    break;
 		    }
 		    if (path[j] || (j - pfsnl - 1) < 1
@@ -404,6 +424,8 @@ ck_file_arg(i, ac, av)
 
 	    } while (mx < nm);
 	}
+	if (!ss)
+	    err = 1;
 	return((int)err);
 }
 
@@ -417,7 +439,6 @@ int
 ctrl_dcache(c)
 	char *c;			/* control string */
 {
-	MALLOC_S len;
 	int rc = 0;
 	
 	if (!c) {
@@ -511,14 +532,13 @@ int
 enter_fd(f)
 	char *f;			/* file descriptor list pointer */
 {
-	char buf[32], c, *cp1, *cp2, *dash;
+	char c, *cp1, *cp2, *dash;
 	int err, hi, lo;
 	char *fc;
-	MALLOC_S len;
 /*
  *  Check for non-empty list and make a copy.
  */
-	if (!f || (len = strlen(f) + 1) < 2) {
+	if (!f || (strlen(f) + 1) < 2) {
 	    (void) fprintf(stderr, "%s: no file descriptor specified\n", Pn);
 	    return(1);
 	}
@@ -546,20 +566,10 @@ enter_fd(f)
 		if (dash) {
 		    if (ckfd_range(cp1, dash, cp2, &lo, &hi))
 			err = 1;
-		    else {
-			while (lo <= hi) {
-			    (void) sprintf(buf, "%d", lo);
-			    if (enter_str_lst("d", buf, &Fdl)) {
-				err = 1;
-				break;
-			    }
-			    lo++;
-			}
-		    }
-		} else {
-		    if (enter_str_lst("d", cp1, &Fdl))
-			err = 1;
-		}
+		    else
+			(void) enter_fd_lst((char *)NULL, lo, hi);
+		} else
+		    (void) enter_fd_lst(cp1, 0, 0);
 	    }
 	    if (c == '\0')
 		break;
@@ -570,6 +580,308 @@ enter_fd(f)
 }
 
 
+/*
+ * enter_fd_lst() - make an entry in the FD list, Fdl
+ */
+
+static void
+enter_fd_lst(nm, lo, hi)
+	char *nm;			/* FD name (none if NULL) */
+	int lo;				/* FD low boundary (if nm NULL) */
+	int hi;				/* FD high boundary (if nm NULL) */
+{
+	char *cp;
+	struct fd_lst *f;
+	int n;
+/*
+ * Allocate an fd_lst entry.
+ */
+	if (!(f = (struct fd_lst *)malloc((MALLOC_S)sizeof(struct fd_lst)))) {
+	   (void) fprintf(stderr, "%s: no space for FD list entry\n", Pn);
+	   Exit(1);
+	}
+	if (nm) {
+
+	/*
+	 * Process an FD name.  First see if it contains only digits; if it
+	 * does, convert them to an integer and set the low and high
+	 * boundaries to the result.
+	 *
+	 * If the name has a non-digit, store it as a string, and set the
+	 * boundaries to impossible values (i.e., low > high).
+	 */
+	    for (cp = nm, n = 0; *cp; cp++) {
+		if (!isdigit((unsigned char)*cp))
+		    break;
+		n = (n * 10) + (int)(*cp - '0');
+	    }
+	    if (*cp) {
+		if (!(f->nm = mkstrcpy(nm, (MALLOC_S *)NULL))) {
+		    (void) fprintf(stderr,
+			"%s: no space for copy of: %s\n", Pn, nm);
+		    Exit(1);
+		}
+		lo = 1;
+		hi = 0;
+	    } else {
+		f->nm = (char *)NULL;
+		lo = hi = n;
+	    }
+	} else
+	    f->nm = (char *)NULL;
+	f->hi = hi;
+	f->lo = lo;
+	f->next = Fdl;
+	Fdl = f;
+}
+
+
+/*
+ * enter_dir() - enter the files of a directory for searching
+ */
+
+#define	EDDEFFNL	128		/* default file name length */
+
+int
+enter_dir(d, descend)
+	char *d;			/* directory path name pointer */
+	int descend;			/* subdirectory descend flag:
+					 *	0 = don't descend
+					 *	1 = descend */
+{
+	char *av[2];
+	dev_t ddev;
+	DIR *dfp;
+	char *dn = (char *)NULL;
+	MALLOC_S dnl, dnamlen;
+	struct DIRTYPE *dp;
+	int en, sl;
+	int fct = 0;
+	char *fp = (char *)NULL;
+	MALLOC_S fpl = (MALLOC_S)0;
+	MALLOC_S fpli = (MALLOC_S)0;
+	struct stat sb;
+/*
+ * Check the directory path; reduce symbolic links; stat(2) it; make sure it's
+ * really a directory.
+ */
+	if (!d || !*d || *d == '+' || *d == '-') {
+	    if (!Fwarn)
+		(void) fprintf(stderr,
+		    "%s: +d not followed by a directory path\n", Pn);
+	    return(1);
+	}
+	if (!(dn = Readlink(d)))
+	    return(1);
+	if (statsafely(dn, &sb)) {
+	    if (!Fwarn) {
+		en = errno;
+		(void) fprintf(stderr, "%s: WARNING: can't stat(", Pn);
+		safestrprt(dn, stderr, 0);
+		(void) fprintf(stderr, "): %s\n", strerror(en));
+	    }
+	    if (dn && dn != d) {
+		(void) free((FREE_P *)dn);
+		dn = (char *)NULL;
+	    }
+	    return(1);
+	}
+	if ((sb.st_mode & S_IFMT) != S_IFDIR) {
+	    if (!Fwarn) {
+		(void) fprintf(stderr, "%s: WARNING: not a directory: ", Pn);
+		safestrprt(dn, stderr, 1);
+	    }
+	    if (dn && dn != d) {
+		(void) free((FREE_P *)dn);
+		dn = (char *)NULL;
+	    }
+	    return(1);
+	}
+	ddev = sb.st_dev;
+/*
+ * Stack the directory and record it in Sfile for searching.
+ */
+	Dstkn = Dstkx = 0;
+	Dstk = (char **)NULL;
+	(void) stkdir(dn);
+	av[0] = (dn == d) ? mkstrcpy(dn, (MALLOC_S)NULL) : dn;
+	av[1] = (char *)NULL;
+	dn = (char *)NULL;
+	if (!ck_file_arg(0, 1, av, 1, 1, &sb)) {
+	    av[0] = (char *)NULL;
+	    fct++;
+	}
+/*
+ * Unstack the next directory and examine it.
+ */
+	while (--Dstkx >= 0) {
+	    if (!(dn = Dstk[Dstkx]))
+		continue;
+	    Dstk[Dstkx] = (char *)NULL;
+	/*
+	 * Open the directory path and prepare its name for use with the
+	 * files in the directory.
+	 */
+	    if (!(dfp = OpenDir(dn))) {
+		if (!Fwarn) {
+		    if ((en = errno) != ENOENT) {
+			(void) fprintf(stderr,
+			    "%s: WARNING: can't opendir(", Pn);
+			safestrprt(dn, stderr, 0);
+			(void) fprintf(stderr, "): %s\n", strerror(en));
+		    }
+	        }
+		(void) free((FREE_P *)dn);
+		dn = (char *)NULL;
+		continue;
+	    }
+	    dnl = strlen(dn);
+	    sl = ((dnl > 0) && (*(dn + dnl - 1) == '/')) ? 0 : 1;
+	/*
+	 * Define space for possible addition to the directory path.
+	 */
+	    fpli = (MALLOC_S)(dnl + sl + EDDEFFNL + 1);
+	    if ((int)fpli > (int)fpl) {
+		fpl = fpli;
+		if (!fp)
+		    fp = (char *)malloc(fpl);
+		else
+		    fp = (char *)realloc(fp, fpl);
+		if (!fp) {
+		    (void) fprintf(stderr,
+			"%s: no space for path to entries in directory: %s\n",
+			Pn, dn);
+		    Exit(1);
+		}
+	    }
+	    (void) sprintf(fp, "%s%s", dn, sl ? "/" : "");
+	    (void) free((FREE_P *)dn);
+	    dn = (char *)NULL;
+	/*
+	 * Read the contents of the directory.
+	 */
+	    for (dp = ReadDir(dfp); dp; dp = ReadDir(dfp)) {
+
+	    /*
+	     * Skip: entries with no inode number;
+	     *	     entries with a zero length name;
+	     *	     ".";
+	     *	     and "..".
+	     */
+		if (!dp->d_ino)
+		    continue;
+
+#if     defined(HASDNAMLEN)
+		dnamlen = (MALLOC_S)dp->d_namlen;
+#else   /* !defined(HASDNAMLEN) */
+		dnamlen = (MALLOC_S)strlen(dp->d_name);
+#endif  /* defined(HASDNAMLEN) */
+
+		if (!dnamlen)
+		    continue;
+		if (dnamlen <= 2 && dp->d_name[0] == '.') {
+		    if (dnamlen == 1)
+			continue;
+		    if (dp->d_name[1] == '.')
+			continue;
+		}
+	    /*
+	     * Form the entry's path name.
+	     */
+		fpli = (MALLOC_S)(dnamlen - (fpl - dnl - sl - 1));
+		if ((int)fpli > 0) {
+		    fpl += fpli;
+		    if (!(fp = (char *)realloc(fp, fpl))) {
+			(void) fprintf(stderr, "%s: no space for: ", Pn);
+			safestrprt(dn, stderr, 0);
+			putc('/', stderr);
+			safestrprtn(dp->d_name, dnamlen, stderr, 1);
+			Exit(1);
+		    }
+		}
+		(void) strncpy(fp + dnl + sl, dp->d_name, dnamlen);
+		fp[dnl + sl + dnamlen] = '\0';
+	    /*
+	     * Lstatsafely() the entry; complain if that fails.
+	     *
+	     * Ignore symbolic links and files not not the directory's device.
+	     *
+	     * Stack entries that represent subdirectories.
+	     */
+		if (lstatsafely(fp, &sb)) {
+		    if ((en = errno) != ENOENT) {
+			if (!Fwarn) {
+			    (void) fprintf(stderr,
+				"%s: WARNING: can't lstat(", Pn);
+			    safestrprt(fp, stderr, 0);
+			    (void) fprintf(stderr, "): %s\n", strerror(en));
+			}
+		    }
+		    continue;
+		}
+		if ((sb.st_mode & S_IFMT) == S_IFLNK || sb.st_dev != ddev)
+		    continue;
+		if (av[0]) {
+		    (void) free((FREE_P *)av[0]);
+		    av[0] = (char *)NULL;
+		}
+		av[0] = mkstrcpy(fp, (MALLOC_S *)NULL);
+		if ((sb.st_mode & S_IFMT) == S_IFDIR && descend)
+
+		/*
+		 * Stack a subdirectory according to the descend argument.
+		 */
+		    stkdir(av[0]);
+	    /*
+	     * Use ck_file_arg() to record the entry for searching.  Force it
+	     * to consider the entry a file, not a file system.
+	     */
+		if (!ck_file_arg(0, 1, av, 1, 1, &sb)) {
+		    av[0] = (char *)NULL;
+		    fct++;
+		}
+	    }
+	    (void) CloseDir(dfp);
+	    if (dn && dn != d) {
+		(void) free((FREE_P *)dn);
+		dn = (char *)NULL;
+	    }
+	}
+/*
+ * Free malloc()'d space.
+ */
+	if (dn && dn != d) {
+	    (void) free((FREE_P *)dn);
+	    dn = (char *)NULL;
+	}
+	if (av[0] && av[0] != fp) {
+	    (void) free((FREE_P *)av[0]);
+	    av[0] = (char *)NULL;
+	}
+	if (fp) {
+	    (void) free((FREE_P *)fp);
+	    fp = (char *)NULL;
+	}
+	if (Dstk) {
+	    (void) free((FREE_P *)Dstk);
+	    Dstk = (char **)NULL;
+	}
+	if (!fct) {
+
+	/*
+	 * Warn if no files were recorded for searching.
+	 */
+	    if (!Fwarn) {
+		(void) fprintf(stderr,
+		    "%s: WARNING: no files found in directory: ", Pn);
+		safestrprt(d, stderr, 1);
+	    }
+	    return(1);
+	}
+	return(0);
+}
+
+
 /*
  * enter_id() - enter PGRP or PID for searching
  */
@@ -620,7 +932,7 @@ enter_id(ty, p)
 
 #if	defined(__STDC__)
 		if (!isdigit((unsigned char)*cp))
-#else
+#else	/* !defined(__STDC__) */
 		if (!isascii(*cp) || ! isdigit((unsigned char)*cp))
 #endif	/* __STDC__ */
 
@@ -687,8 +999,8 @@ int
 enter_network_address(na)
 	char *na;			/* Internet address string pointer */
 {
-	unsigned char *ap;
-	int ep, pr, sp;
+	int ae, i, pr;
+	int ep = -1;
 	struct hostent *he = (struct hostent *)NULL;
 	char *hn = (char *)NULL;
 	MALLOC_S l;
@@ -698,21 +1010,22 @@ enter_network_address(na)
 	int pu = 0;
 	struct servent *se, *se1;
 	char *sn = (char *)NULL;
+	int sp = -1;
 	MALLOC_S snl = 0;
 
+#if	defined(HASIPv6)
+	char *cp;
+#endif	/* defined(HASIPv6) */
+
 	if (!na) {
 	    (void) fprintf(stderr, "%s: no network address specified\n", Pn);
 	    return(1);
 	}
-/*
- * Initialize network address structure.
- */
-	n.proto = (char *)NULL;
-	n.a[0] = n.a[1] = n.a[2] = n.a[3] = 0;
 /*
  * Process protocol name, optionally followed by a '@' and a host name or
  * Internet address, or a ':' and a service name or port number.
  */
+	zeromem((char *)&n, sizeof(n));
 	wa = na;
 	if (*wa && *wa != '@' && *wa != ':') {
 	    for (p = wa; *wa && *wa != '@' && *wa != ':'; wa++)
@@ -754,8 +1067,9 @@ nwad_exit:
 	    }
 	}
 /*
- * Process an Internet address (1.2.3.4) or host name, preceded by a '@'
- * and optionally followed by a colon and a service name or port number.
+ * Process an IPv4 address (1.2.3.4), IPv6 address ([1:2:3:4:5:6:7:8]),
+ * or host name, preceded by a '@' and optionally followed by a colon
+ * and a service name or port number.
  */
 	if (*wa == '@') {
 	    wa++;
@@ -767,10 +1081,11 @@ unacc_address:
 		safestrprt(na, stderr, 1);
 		goto nwad_exit;
 	    }
-	    if (*wa < '0' || *wa > '9') {
-	/*
-	 * Assemble host name.
-	 */
+	    if ((*wa < '0' || *wa > '9') && *wa != '[') {
+
+	    /*
+	     * Assemble host name.
+	     */
 		for (p = wa; *p && *p != ':'; p++)
 		    ;
 		if ((l = p - wa)) {
@@ -782,53 +1097,97 @@ unacc_address:
 			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
-		    if ((he = gethostbyname(hn)) == NULL) {
+
+#if	defined(HASIPv6)
+		    n.af = AF_INET6;
+#else	/* !defined(HASIPv6) */
+		    n.af = AF_INET;
+#endif	/* defined(HASIPv6) */
+
+		    he = lkup_hostnm(hn, &n);
+
+#if	defined(HASIPv6)
+		    if (!he) {
+			n.af = AF_INET;
+			he = lkup_hostnm(hn, &n);
+		    }
+#endif	/* defined(HASIPv6) */
+		
+		    if (!he) {
 			fprintf(stderr, "%s: unknown host name (%s) in ",
 			    Pn, hn);
 			safestrprt(na, stderr, 1);
 			goto nwad_exit;
 		    }
-		    ap = (unsigned char *)he->h_addr;
-		    n.a[0] = *ap++;
-		    n.a[1] = *ap++;
-		    n.a[2] = *ap++;
-		    n.a[3] = *ap;
 		}
 		wa = p;
+	    } else if (*wa == '[') {
+
+#if	defined(HASIPv6)
+	    /*
+	     * Assemble IPv6 address.
+	     */
+		if (!(cp = strrchr(++wa, ']')))
+		    goto unacc_address;
+		*cp = '\0';
+		i = inet_pton(AF_INET6, wa, (void *)&n.a);
+		*cp = ']';
+		if (i != 1)
+		    goto unacc_address;
+		for (ae = i = 0; i < MAX_AF_ADDR; i++) {
+		    if ((ae |= n.a[i]))
+			break;
+		}
+		if (!ae)
+		    goto unacc_address;
+		if (IN6_IS_ADDR_V4MAPPED((struct in6_addr *)&n.a[0])) {
+		    for (i = 0; i < 4; i++) {
+			n.a[i] = n.a[i+12];
+		    }
+		    n.af = AF_INET;
+		} else
+		    n.af = AF_INET6;
+		wa = cp + 1;
+#else	/* !defined(HASIPv6) */
+		(void) fprintf(stderr,
+		    "%s: unsupported IPv6 address in ", Pn);
+		safestrprt(na, stderr, 1);
+		goto nwad_exit;
+#endif	/* defined(HASIPv6) */
+
+
 	    } else {
-	/*
-	 * Assemble Internet address.
-	 */
-		for (l = 0; *wa; wa++) {
+
+	    /*
+	     * Assemble IPv4 address.
+	     */
+		for (i = 0; *wa; wa++) {
 		    if (*wa == ':')
 			break;
 		    if (*wa == '.') {
-			l++;
-			if (l > 3)
+			i++;
+			if (i >= MIN_AF_ADDR)
 			    break;
 			continue;
 		    }
 		    if (*wa < '0' || *wa > '9')
 			goto unacc_address;
-		    n.a[l] = (10 * n.a[l]) + *wa - '0';
-		    if (n.a[l] > 255)
+		    ae = (10 * n.a[i]) + *wa - '0';
+		    if (ae > 255)
 			goto unacc_address;
+		    n.a[i] = (unsigned char)ae;
 		}
-		if (l != 3
-		||  (n.a[0] == 0 && n.a[1] == 0 && n.a[2] == 0 && n.a[3] == 0))
+		if (i != (MIN_AF_ADDR - 1)
+		||  (!n.a[0] && !n.a[1] && !n.a[2] && !n.a[3]))
 		    goto unacc_address;
+		n.af = AF_INET;
 	    }
 	}
 /*
  * If there is no port number, enter the address.
  */
-	if (!*wa) {
-	    if (enter_nwad(&n, -1, -1, na, he))
-		goto nwad_exit;
-	    if (sn)
-		(void) free((FREE_P *)sn);
-	    return(0);
-	}
+	if (!*wa)
+	    goto nwad_enter;
 /*
  * Process a service name or port number list, preceded by a colon.
  *
@@ -962,8 +1321,28 @@ unacc_port:
 	/*
 	 * Enter completed port or port range specification.
 	 */
-	    if (enter_nwad(&n, sp, ep, na, he))
-		goto nwad_exit;
+
+nwad_enter:
+
+	    for (i = 1; i;) {
+		if (enter_nwad(&n, sp, ep, na, he))
+		    goto nwad_exit;
+
+#if	defined(HASIPv6)
+	    /*
+	     * If IPv6 is enabled, a host name was specified, and the
+	     * associated * address is for the AF_INET6 address family,
+	     * try to get and address for the AF_INET family, too.
+	     */
+		if (hn && (n.af == AF_INET6)) {
+		    n.af = AF_INET;
+		    if ((he = lkup_hostnm(hn, &n)))
+			continue;
+		}
+#endif	/* defined(HASIPv6) */
+
+		i = 0;
+	    }
 	    if (!*wa)
 		break;
 	}
@@ -1014,7 +1393,15 @@ enter_nwad(n, sp, ep, s, he)
 	 * list.
 	 */
 	    if (!nc.proto
-	    &&  nc.a[0] == 0 && nc.a[1] == 0 && nc.a[2] == 0 && nc.a[3] == 0
+	    &&  !nc.a[0] && !nc.a[1] && !nc.a[2] && !nc.a[3]
+
+#if	defined(HASIPv6)
+	    &&  (nc.af != AF_INET6
+	    ||   (!nc.a[4]  && !nc.a[5]  && !nc.a[6]  && !nc.a[7]
+	    &&    !nc.a[8]  && !nc.a[9]  && !nc.a[10] && !nc.a[11]
+	    &&    !nc.a[12] && !nc.a[13] && !nc.a[14] && !nc.a[15]))
+#endif	/* defined(HASIPv6) */
+
 	    &&  sp == -1) {
 		(void) fprintf(stderr,
 		    "%s: incomplete Internet address specification: ", Pn);
@@ -1059,10 +1446,26 @@ enter_nwad(n, sp, ep, s, he)
 		break;
 	    if (!(ap = (unsigned char *)he->h_addr_list[ac++]))
 		break;
+
+#if	defined(HASIPv6)
+	    {
+		int i;
+
+		for (i = 0;
+		     (i < (he->h_length - 1)) && (i < (MAX_AF_ADDR - 1));
+		     i++)
+		{
+		    nc.a[i] = *ap++;
+		}
+		nc.a[i] = *ap;
+	    }
+#else	/* !defined(HASIPv6) */
 	    nc.a[0] = *ap++;
 	    nc.a[1] = *ap++;
 	    nc.a[2] = *ap++;
 	    nc.a[3] = *ap;
+#endif	/* defined(HASIPv6) */
+
 	}
 	return(0);
 }
@@ -1082,7 +1485,7 @@ enter_str_lst(opt, s, lp)
 	MALLOC_S len;
 	struct str_lst *lpt;
 
-	if (!s || *s == '-') {
+	if (!s || *s == '-' || *s == '+') {
 	    (void) fprintf(stderr, "%s: missing %s option value\n",
 		Pn, opt);
 	    return(1);
@@ -1115,11 +1518,12 @@ int
 enter_uid(u)
 	char *u;			/* User IDentifier string pointer */
 {
-	int err, excl, i, j, lnml, nn;
+	int err, i, j, lnml, nn;
+	unsigned char excl;
 	MALLOC_S len;
-	char lnm[LOGINML], *lp;
+	char lnm[LOGINML+1], *lp;
 	struct passwd *pw;
-	char *s;
+	char *s, *st;
 	uid_t uid;
 
 	if (!u) {
@@ -1131,12 +1535,20 @@ enter_uid(u)
 	/*
 	 * Assemble next User IDentifier.
 	 */
-	    for (excl = i = j = lnml = nn = uid = 0; *s && *s != ','; i++, s++)
+	    for (excl = i = j = lnml = nn = uid = 0, st = s;
+		 *s && *s != ',';
+		 i++, s++)
 	    {
-		if (j >= LOGINML-1) {
-		    (void) fprintf(stderr, "%s: illegal UID in ", Pn);
-		    safestrprt(u, stderr, 1);
-		    return(1);
+		if (lnml >= LOGINML) {
+		    while (*s && *s != ',') {
+			s++;
+			lnml++;
+		    }
+		    (void) fprintf(stderr,
+			"%s: -u login name > %d characters: ", Pn, LOGINML);
+		    safestrprtn(st, lnml, stderr, 1);
+		    err = j = 1;
+		    break;
 		}
 		if (i == 0 && *s == '^') {
 		    excl = 1;
@@ -1158,6 +1570,8 @@ enter_uid(u)
 	    }
 	    if (*s)
 		s++;
+	    if (j)
+		continue;
 	    if (nn) {
 	       lnm[lnml++] = '\0';
 		if ((pw = getpwnam(lnm)) == NULL) {
@@ -1235,3 +1649,65 @@ enter_uid(u)
 	}
 	return(err);
 }
+
+
+/*
+ * lkup_hostnm() - look up host name
+ */
+
+static struct hostent *
+lkup_hostnm(hn, n)
+	char *hn;			/* host name */
+	struct nwad *n;			/* network address destination */
+{
+	unsigned char *ap;
+	struct hostent *he;
+	int ln;
+/*
+ * Get hostname structure pointer.  Return NULL if there is none.
+ */
+
+#if	defined(HASIPv6)
+	he = gethostbyname2(hn, n->af);
+#else	/* !defined(HASIPv6) */
+	he = gethostbyname(hn);
+#endif	/* defined(HASIPv6) */
+
+	if (!he)
+	    return(he);
+/*
+ * Copy first hostname structure address to destination structure.
+ */
+
+#if	defined(HASIPv6)
+	if (n->af != he->h_addrtype)
+	    return((struct hostent *)NULL);
+	if (n->af == AF_INET6) {
+
+	/*
+	 * Copy an AF_INET6 address.
+	 */
+	    if (he->h_length > MAX_AF_ADDR)
+		return((struct hostent *)NULL);
+	    (void) memcpy((void *)&n->a[0], (void *)he->h_addr, he->h_length);
+	    if ((ln = MAX_AF_ADDR - he->h_length) > 0)
+		zeromem((char *)&n->a[he->h_length], ln);
+	    return(he);
+	}
+#endif	/* defined(HASIPv6) */
+
+/*
+ * Copy an AF_INET address.
+ */
+	if (he->h_length != 4)
+	    return((struct hostent *)NULL);
+	ap = (unsigned char *)he->h_addr;
+	n->a[0] = *ap++;
+	n->a[1] = *ap++;
+	n->a[2] = *ap++;
+	n->a[3] = *ap;
+	if ((ln = MAX_AF_ADDR - 4) > 0)
+	    zeromem((char *)&n->a[4], ln);
+	return(he);
+
+}
