@@ -32,18 +32,18 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.6 98/02/16 19:47:48 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.13 2000/06/09 13:48:35 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
-_PROTOTYPE(static void process_lock,(struct lockf *lp));
+_PROTOTYPE(static void process_lock,(KA_T lp));
 
 #if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
@@ -62,21 +62,20 @@ getmemsz(pid)
 	struct vmspace vm;
 
 	for (n = 0, p = P; n < Np; n++, p++) {
-		if (p->P_PID == pid) {
-			if (!p->P_VMSPACE
-			||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
-				return;
-			Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
-							     + vm.vm_ssize);
-			Lf->sz_def = 1;
-			return;
-		}
+	    if (p->P_PID == pid) {
+		if (!p->P_VMSPACE
+		||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
+		    return;
+		Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize+vm.vm_dsize+vm.vm_ssize);
+		Lf->sz_def = 1;
+		return;
+	    }
 	}
 }
 #endif	/* defined(HASPROCFS) */
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 /*
  * lkup_dev_tty() - look up /dev/tty
  */
@@ -119,7 +118,7 @@ lkup_dev_tty_again:
 
 	return(-1);
 }
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
 /*
@@ -128,9 +127,10 @@ lkup_dev_tty_again:
 
 static void
 process_lock(lp)
-	struct lockf *lp;		/* kernel lockf structure */
+	KA_T lp;			/* kernel lockf structure */
 {
-	struct lockf lf, *lfp;
+	struct lockf lf;
+	KA_T lfp;
 	int lt;
 
 	if (!lp)
@@ -140,12 +140,12 @@ process_lock(lp)
  */
 	lfp = lp;
 	do {
-	    if (kread((KA_T)lfp, (char *)&lf, sizeof(lf)))
+	    if (kread(lfp, (char *)&lf, sizeof(lf)))
 		break;
 	    lt = 0;
 	    switch (lf.lf_flags & (F_FLOCK|F_POSIX)) {
 	    case F_FLOCK:
-		if (Cfp && (struct file *)lf.lf_id == Cfp)
+		if (Cfp && (KA_T)lf.lf_id == Cfp)
 		    lt = 1;
 		break;
 	    case F_POSIX:
@@ -166,7 +166,7 @@ process_lock(lp)
 	    else if (lf.lf_type == (F_RDLCK | F_WRLCK))
 		Lf->lock = 'u';
 	    return;
-	} while ((lfp = lf.lf_next) && lfp != lp);
+	} while ((lfp = (KA_T)lf.lf_next) && lfp != lp);
 }
 
 
@@ -176,13 +176,12 @@ process_lock(lp)
 
 void
 process_node(va)
-	caddr_t va;			/* vnode kernel space address */
+	KA_T va;			/* vnode kernel space address */
 {
 	struct iso_node cd;
 	u_int cn;
 	dev_t dev;
 	struct denode dn;
-	char dev_ch[32];
 	int lt;
 	struct inode i;
 	struct mfsnode m;
@@ -196,9 +195,13 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	struct fdescnode f;
+
+# if	HASFDESCFS==1
 	static dev_t f_tty_dev;
 	static unsigned long f_tty_ino;
 	static int f_tty_s = 0;
+# endif	/* HASFDESCFS==1 */
+
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASPROCFS)
@@ -209,20 +212,27 @@ process_node(va)
 /*
  * Read the vnode.
  */
-	if ( ! va) {
+	if (!va) {
 	    enter_nm("no vnode address");
 	    return;
 	}
 	v = &vb;
-	if (readvnode((caddr_t)va, v)) {
+	if (readvnode(va, v)) {
 	    enter_nm(Namech);
 	    return;
         }
 
-# if	defined(HASNCACHE)
+#if	defined(HASNCACHE)
+	Lf->na = va;
+# if	defined(HASNCAPID)
 	Lf->id = v->v_id;
-	Lf->na = (unsigned long)va;
-# endif	/* defined(HASNCACHE) */
+# endif	/* defined(HASNCAPID) */
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fna = va;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Get the vnode type.
@@ -230,7 +240,7 @@ process_node(va)
 	if (!v->v_mount)
 	    vfs = (struct l_vfs *)NULL;
 	else {
-	    vfs = readvfs(v->v_mount);
+	    vfs = readvfs((KA_T)v->v_mount);
 	    if (vfs) {
 
 #if	BSDIV<30000
@@ -305,35 +315,38 @@ process_node(va)
  */
 	switch (v->v_tag) {
 	case VT_ISOFS:
-	    if (kread((KA_T)v->v_data, (char *)&cd, sizeof(cd)) != 0) {
-		(void) sprintf(Namech, "can't read cd9660_node at: %x",
-		    v->v_data);
+	    if (kread((KA_T)v->v_data, (char *)&cd, sizeof(cd))) {
+		(void) sprintf(Namech, "can't read cd9660_node at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    nty = CDFSNODE;
-	    (void) process_lock(cd.i_lockf);
+	    (void) process_lock((KA_T)cd.i_lockf);
 	    break;
 	case VT_MFS:
-	    if (kread((KA_T)v->v_data, (char *)&m, sizeof(m)) != 0) {
-		(void) sprintf(Namech, "can't read mfsnode at: %x", v->v_data);
+	    if (kread((KA_T)v->v_data, (char *)&m, sizeof(m))) {
+		(void) sprintf(Namech, "can't read mfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    nty = MFSNODE;
 	    break;
 	case VT_MSDOSFS:
-	    if (kread((KA_T)v->v_data, (char *)&dn, sizeof(dn)) != 0) {
-		(void) sprintf(Namech, "can't read denode at: %x", v->v_data);
+	    if (kread((KA_T)v->v_data, (char *)&dn, sizeof(dn))) {
+		(void) sprintf(Namech, "can't read denode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    nty = PCFSNODE;
-	    (void) process_lock(dn.de_lockf);
+	    (void) process_lock((KA_T)dn.de_lockf);
 	    break;
 	case VT_NFS:
-	    if (kread((KA_T)v->v_data, (char *)&n, sizeof(n)) != 0) {
-		(void) sprintf(Namech, "can't read nfsnode at: %x", v->v_data);
+	    if (kread((KA_T)v->v_data, (char *)&n, sizeof(n))) {
+		(void) sprintf(Namech, "can't read nfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
@@ -342,21 +355,21 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	case VT_FDESC:
-		if (kread((KA_T)v->v_data, (char *)&f, sizeof(f)) != 0) {
-			(void) sprintf(Namech, "can't read fdescnode at: %x",
-				v->v_data);
-			enter_nm(Namech);
-			return;
-		}
-		nty = FDESCNODE;
-		break;
+	    if (kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
+		(void) sprintf(Namech, "can't read fdescnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = FDESCNODE;
+	    break;
 #endif	/* defined(HASFDESCFS) */
 
-
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
-	    if (kread((KA_T)v->v_data, (char *)&p, sizeof(p)) != 0) {
-		(void) sprintf(Namech, "can't read pfsnode at: %x", v->v_data);
+	    if (kread((KA_T)v->v_data, (char *)&p, sizeof(p))) {
+		(void) sprintf(Namech, "can't read pfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
@@ -365,13 +378,14 @@ process_node(va)
 #endif	/* defined(HASPROCFS) */
 
 	case VT_UFS:
-	    if (kread((KA_T)v->v_data, (char *)&i, sizeof(i)) != 0) {
-		(void) sprintf(Namech, "can't read inode at: %x", v->v_data);
+	    if (kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
+		(void) sprintf(Namech, "can't read inode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    nty = INODE;
-	    (void) process_lock(i.i_lockf);
+	    (void) process_lock((KA_T)i.i_lockf);
 	    break;
 	}
 /*
@@ -401,22 +415,28 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	case FDESCNODE:
-		if (f.fd_link
-		&&  kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1) == 0)
-		{
-		    Namech[sizeof(Namech) - 1] = '\0';
-		    break;
-		}
-		if (f.fd_type == Fctty) {
-		    if (f_tty_s == 0)
-			f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
-		    if (f_tty_s == 1) {
-			dev = f_tty_dev;
-			Lf->inode = f_tty_ino;
-			Lf->inp_ty = 1;
-		    }
-		}
+
+# if	defined(HASFDLINK)
+	    if (f.fd_link
+	    &&  !kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1)) {
+		Namech[sizeof(Namech) - 1] = '\0';
 		break;
+	    }
+# endif	/* defined(HASFDLINK)
+
+# if	HASFDESCFS==1
+	    if (f.fd_type == Fctty) {
+		if (f_tty_s == 0)
+		    f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
+		if (f_tty_s == 1) {
+		    dev = f_tty_dev;
+		    Lf->inode = f_tty_ino;
+		    Lf->inp_ty = 1;
+		}
+	    }
+	    break;
+# endif	/* HASFDESCFS==1 */
+
 #endif	/* defined(HASFDESCFS) */
 
 	case INODE:
@@ -461,7 +481,6 @@ process_node(va)
 #endif	/* defined(HASPROCFS) */
 
 	}
-
 /*
  * Obtain the file size.
  */
@@ -523,6 +542,41 @@ process_node(va)
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch (Ntype) {
+	    case N_CDFS:
+		Lf->nlink = (long)cd.inode.iso_links;
+		Lf->nlink_def = 1;
+		break;
+	    case N_FIFO:		/* no link count? */
+		break;
+	    case N_PCFS:		/* no link count? */
+		break;
+	    case N_NFS:
+		if (nty == NFSNODE) {
+		    Lf->nlink = (long)n.n_vattr.va_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+
+# if	defined(HASPROCFS)
+	    case N_PROC:		/* no link count? */
+		break;
+# endif	/* defined(HASPROCFS) */
+
+	    case N_REGLR:
+		if (nty == INODE) {
+		    Lf->nlink = (long)i.i_din.di_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS file selection.
  */
@@ -539,7 +593,6 @@ process_node(va)
  * Format the vnode type, and possibly the device name.
  */
 	switch (type) {
-
 	case VNON:
 	    ty ="VNON";
 	    break;
@@ -603,8 +656,7 @@ process_node(va)
 	else if (nty == MFSNODE) {
 	    Lf->dev_def = 0;
 	    (void) sprintf(Namech, "%#x", m.mfs_baseoff);
-	    (void) strcpy(dev_ch, "    memory");
-	    enter_dev_ch(dev_ch);
+	    enter_dev_ch("memory");
 	}
 
 #if	defined(HASPROCFS)
@@ -706,7 +758,7 @@ process_node(va)
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named((char *)NULL, type))
+	    if (Sfile && is_file_named((char *)NULL, (type == VCHR) ? 1: 0))
 		Lf->sf |= SELNM;
 	}
 /*
