@@ -32,16 +32,16 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.11 98/03/06 14:44:19 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.17 2000/06/09 13:48:52 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 #if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
@@ -79,7 +79,7 @@ getmemsz(pid)
 #endif	/* defined(HASPROCFS) */
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 /*
  * lkup_dev_tty() - look up /dev/tty
  */
@@ -122,7 +122,7 @@ lkup_dev_tty_again:
 
 	return(-1);
 }
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
 /*
@@ -152,14 +152,19 @@ process_node(va)
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
 	unsigned long iso_ino, iso_sz;
+	long iso_nlink;
 	int iso_stat = 0;
 #endif	/* defined(HAS9660FS) */
 
 #if	defined(HASFDESCFS)
 	struct fdescnode f;
+
+# if	HASFDESCFS==1
 	static dev_t f_tty_dev;
 	static unsigned long f_tty_ino;
 	static int f_tty_s = 0;
+# endif	/* HASFDESCFS==1 */
+
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASKERNFS)
@@ -196,10 +201,17 @@ process_node(va)
 	    return;
 	}
 
-# if	defined(HASNCACHE)
+#if	defined(HASNCACHE)
+	Lf->na = va;
+# if	defined(HASNCAPID)
 	Lf->id = v->v_id;
-	Lf->na = (unsigned long)va;
-# endif	/* defined(HASNCACHE) */
+# endif	/* defined(HASNCAPID) */
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fna = va;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Get the vnode type.
@@ -244,7 +256,7 @@ process_node(va)
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
 	    if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
-		  (SZOFFTYPE *)&iso_sz))
+		  &iso_nlink, (SZOFFTYPE *)&iso_sz))
 	    {
 		(void) sprintf(Namech, "can't read iso_node at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL));
@@ -440,11 +452,16 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	case FDESCNODE:
+
+# if	defined(HASFDLINK)
 	    if (f.fd_link
 	    &&  !kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1)) {
 		Namech[sizeof(Namech) - 1] = '\0';
 		break;
 	    }
+# endif	/* defined(HASFDLINK) */
+
+# if	HASFDESCFS==1
 	    if (f.fd_type == Fctty) {
 		if (f_tty_s == 0)
 		    f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
@@ -455,6 +472,7 @@ process_node(va)
 		}
 	    }
 	    break;
+# endif	/* HASFDESCFS==1 */
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASEXT2FS)
@@ -659,6 +677,60 @@ process_node(va)
 		break;
 	    }
 	}
+/*
+ * Record the link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+
+#if	defined(HAS9660FS)
+	    case N_CDFS:
+		if (iso_stat) {
+		    Lf->nlink = iso_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+#endif	/* defined(HAS9660FS) */
+
+#if	defined(HASKERNFS)
+	    case N_KERN:
+		if (ksbs) {
+		    Lf->nlink = (long)ksb.st_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+#endif	/* defined(HASKERNFS) */
+
+	    case N_NFS:
+		if (nty == NFSNODE) {
+		    Lf->nlink = (long)NVATTR.va_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+	    case N_REGLR:
+		if (nty == INODE) {
+
+# if	defined(HASI_FFS)
+		    Lf->nlink = (long)i.i_ffs_nlink;
+# else	/* !defined(HASI_FFS) */
+		    Lf->nlink = (long)i.i_nlink;
+# endif	/* defined(HASI_FFS) */
+
+		    Lf->nlink_def = 1;
+		}
+
+#if	defined(HASEXT2FS)
+		else if (nty == EXT2NODE) {
+		    Lf->nlink - (long)i.i_e2fs_nlink;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* defined(HASEXT2FS) */
+
+		break;
+	    }
+	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS file selection.
  */
@@ -859,7 +931,7 @@ process_node(va)
 		ns = 1;
 	    } else
 		ns = 0;
-	    if (Sfile && is_file_named(NULL, type))
+	    if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
 		Lf->sf |= SELNM;
 	    if (ns)
 		Namech[0] = '\0';
