@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.15 2000/03/23 14:21:33 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.16 2000/05/31 13:04:05 abe Exp $";
 #endif
 
 
@@ -192,8 +192,16 @@ process_socket(sa)
 	     */
 		if (!s.so_pcb
 		||  kread((KA_T)s.so_pcb, (char *)&inp, sizeof(inp))) {
-		    (void) sprintf(Namech, "can't read inpcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    if (!s.so_pcb) {
+			(void) strcpy(Namech, "no PCB");
+			if (s.so_state & SS_CANTSENDMORE)
+			    (void) strcpy(endnm(), ", CANTSENDMORE");
+			if (s.so_state & SS_CANTRCVMORE)
+			    (void) strcpy(endnm(), ", CANTRCVMORE");
+		    } else {
+			(void) sprintf(Namech, "can't read inpcb at %s",
+			    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    }
 		    enter_nm(Namech);
 		    return;
 		}
