@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.8 98/02/13 12:44:05 abe Exp $
+ * $Id: dlsof.h,v 1.13 2000/06/25 17:34:16 abe Exp $
  */
 
 
@@ -58,6 +58,11 @@
 #include <setjmp.h>
 #include <signal.h>
 #include <string.h>
+
+#if	TCPV>=450
+#include <stddef.h>
+#endif	/* TCPV>=450 */
+
 #include <stdlib.h>
 #include <nlist.h>
 #include <sys/flock.h>
@@ -80,7 +85,20 @@
 #include <sys/snode.h>
 #include <sys/sysmacros.h>
 #include <sys/swap.h>
+
+# if	TCPV>=450
+/*
+ * Define _INTCP for TCP versions 4.5 and above before #include'ing <sys/un.h>
+ * so the kernel's definition of sockaddr_un will be activated.
+ */
+
+#define	_INTCP	1
 #include <sys/un.h>
+#undef	_INTCP
+# else	/* TCPV<450 */
+#include <sys/un.h>
+# endif	/* TCPV>=450 */
+
 #include <sys/var.h>
 #include <sys/vfs.h>
 #include <sys/ufsmount.h>
@@ -106,6 +124,10 @@
 # endif	/* PTXV<400 */
 
 # if	defined(HAS_NFS)
+#  if	PTXV>=451
+#undef	UDPMSGSIZE
+#include <sys/pathname.h>
+#  endif	/* PTXV>=451 */
 #include <nfs/nfs.h>
 #include <nfs/nfs_clnt.h>
 
@@ -135,27 +157,28 @@
 #include <sys/tihdr.h>
 #include <sys/tiuser.h>
 
-# if	PTXV>=421
+# if	TCPV>=420
 #include <sys/unpcb.h>
-# endif	/* PTXV>=421 */
-
-# if	PTXV>=413
 #include <sys/domain.h>
 #include <sys/protosw.h>
-#  if	PTXV>=430
-#define	_INKERNEL	1
-#  else	/* PTXV<430 */
-#define	INKERNEL	1
-#  endif	/* PTXV>=430 */
+# endif	/* TCPV>=420 */
+
+# if	TCPV>=440
+#define	_INKERNEL	1		/* for <sys/sockmod.h> */
+# else	/* TCPV<440 */
+#  if	TCPV>=420
+#define	INKERNEL	1		/* for <sys/sockmod.h> */
+#  endif	/* TCPV>=420 */
+# endif	/* TCPV>=440 */
 #include <sys/sockmod.h>
-#  if	PTXV>=430
+
+# if	TCPV>=440
 #undef	_INKERNEL
-#  else	/* PTXV<430 */
+# else	/* TCPV<440 */
+#  if	TCPV>=420
 #undef	INKERNEL
-#  endif	/* PTXV>=430 */
-# else	/* PTXV<413 */
-#include <sys/sockmod.h>
-# endif	/* PTXV>=413 */
+#  endif	/* TCPV>=420 */
+# endif	/* TCPV>=440 */
 
 #include <sys/socketvar.h>
 #include <sys/user.h>
@@ -164,9 +187,9 @@
 #undef _INKERNEL
 # endif	/* PTXV<400 */
 
-# if	PTXV>=413
+# if	TCPV>=420
 #include <netinet/ip.h>
-# endif	/* PTXV>=413 */
+# endif	/* TCPV>=420 */
 
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
@@ -177,11 +200,36 @@
 #include <netinet/tcp_var.h>
 
 
+# if	PTXV>=450
+/*
+ * This definition for dnlc_t should appear in <sys/dnlc.h>, but doesn't.
+ *
+ * Laurent Montaron <lpm@sequent.com> filed PR #250929 requesting that the
+ * dnlc_t definition be moved from vfs_dnlc.c tp <sys/dnlc.h>, but as of
+ * PTX 4.5.1 that had not been done.
+ */
+
+typedef struct {
+	lock_t		dnlc_mutex;
+	struct nc_hash	*dnlc_hash;
+	struct ncache	*dnlc_cache;
+	struct ncache	*dnlc_lru;
+	long		dnlc_pad[12];
+} dnlc_t;
+# endif	/* PTXV>=450 */
+
+
 #define	COMP_P		const void
 #define QSORT_P		void
 
-#define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		off_t
+#define DEVINCR		1024		/* device table malloc() increment */
+
+# if	PTXV>=444
+typedef	vaddr_t		KA_T;		/* unsigned for XKVA addresses */
+# else	/* PTXV<444 */
+typedef	off_t		KA_T;
+# endif	/* PTXV>=444 */
+
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
@@ -210,18 +258,12 @@
  * Global storage definitions (including their structure definitions)
  */
 
-struct clone {
-	int dx;				/* index of device entry in Devtp[] */
-	struct clone *next;		/* forward link */
-};
-extern struct clone *Clone;
-
-extern int CloneMajor;			/* clone major device number */
-extern int HaveCloneMajor;		/* CloneMajor status */
+extern int CloneMaj;			/* clone major device number */
+extern int HaveCloneMaj;		/* CloneMaj status */
 extern int Kd;
 
 struct l_vfs {
-	struct vfs *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
@@ -248,6 +290,8 @@ struct mounts {
         struct mounts *next;    	/* forward link */
 };
 
+#define X_DNLCA		"dnlc_array"
+#define	X_DNLCM		"dnlc_mask"
 #define	X_NCACHE	"ncache"
 #define	X_NCSIZE	"ncsize"
 #define	NL_NAME		n_name
@@ -286,6 +330,16 @@ extern int Swap;
 #define	CVFS_DEVSAVE 	1
 
 
+/*
+ * Definitions for dvch.c, isfn.c, and rdev.c
+ */
+
+#define	CLONEMAJ	CloneMaj	/* clone major variable name */
+#define	HAS_STD_CLONE	2		/* has standard clone structure, but
+					 * doesn't cache clone devices */
+#define	HAVECLONEMAJ	HaveCloneMaj	/* clone major status variable name */
+
+
 /*
  * Definitions for rnch.c
  */
