@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.4 97/06/12 18:46:17 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.13 2000/06/25 17:34:16 abe Exp $";
 #endif
 
 
@@ -50,13 +50,17 @@ static char *rcsid = "$Id: dnode.c,v 1.4 97/06/12 18:46:17 abe Exp $";
 
 
 #if	PTXV>=400
-_PROTOTYPE(static void ent_fa,(caddr_t *a1, caddr_t *a2, char *d));
+_PROTOTYPE(static void ent_fa,(KA_T *a1, KA_T *a2, char *d));
 #endif	/* PTXV>=400 */
 
 _PROTOTYPE(static struct l_dev * finddev,(dev_t *dev, int stream));
 _PROTOTYPE(static int isip,(char *mn));
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 
+#if	TCPV>=450
+_PROTOTYPE(static int read_vl_un,(KA_T addr, int len));
+#endif	/* TCPV>=450 */
+
 
 struct iptab_proto {
 	char *mn;			/* module name */
@@ -77,25 +81,26 @@ struct iptab_proto {
 
 static void
 ent_fa(a1, a2, d)
-	caddr_t *a1;			/* first fattach address (NULL OK) */
-	caddr_t *a2;			/* second fattach address */
+	KA_T *a1;			/* first fattach address (NULL OK) */
+	KA_T *a2;			/* second fattach address */
 	char *d;			/* direction ("->" or "<-") */
 {
-	char buf[64], *cp;
+	char buf[128], *cp;
 	MALLOC_S len;
 
 	if (Lf->nma)
-		return;
+	    return;
 	if (!a1)
-		(void) sprintf(buf, "(FA:%s%#x)", d, *a2);
+	    (void) sprintf(buf, "(FA:%s%s)", d, print_kptr(*a2, (char *)NULL));
 	else
-		(void) sprintf(buf, " (FA:%#x%s%#x)", *a1, d, *a2);
+	    (void) sprintf(buf, " (FA:%s%s%s)", print_kptr(*a1, (char *)NULL),
+		d, print_kptr(*a2, (char *)NULL));
 	len = strlen(buf) + 1;
 	if ((cp = (char *)malloc(len)) == NULL) {
-		(void) fprintf(stderr,
-			"%s: no space for fattach addresses at PID %d, FD %s\n",
-			Pn, Lp->pid, Lf->fd);
-		Exit(1);
+	    (void) fprintf(stderr,
+		"%s: no space for fattach addresses at PID %d, FD %s\n",
+		Pn, Lp->pid, Lf->fd);
+	    Exit(1);
 	}
 	(void) strcpy(cp, buf);
 	Lf->nma = cp;
@@ -103,7 +108,6 @@ ent_fa(a1, a2, d)
 #endif	/* PTXV>=400 */
 
 
-
 /*
  * finddev() - look up device by device number
  */
@@ -158,8 +162,8 @@ isip(mn)
 	int i;
 
 	for (i = 0; IPtab[i].mn; i++) {
-		if (strcasecmp(mn, IPtab[i].mn) == 0)
-			return(i);
+	    if (strcasecmp(mn, IPtab[i].mn) == 0)
+		return(i);
 	}
 	return(-1);
 }
@@ -174,7 +178,8 @@ isvlocked(va)
 	struct vnode *va;		/* local vnode address */
 {
 
-	struct filock f, *flf, *flp;
+	struct filock f;
+	KA_T flf, flp;
 	int l;
 
 #if	PTXV>=410
@@ -182,18 +187,18 @@ isvlocked(va)
 #endif	/* PTXV<410 */
 
 #if	PTXV<410
-	if (!(flf = va->v_filocks))
+	if (!(flf = (KA_T)va->v_filocks))
 	    return(' ');
 #else	/* PTXV>=410 */
 	if (!va->v_recp
 	||  kread((KA_T)va->v_recp, (char *)&vl, sizeof(vl))
-	||  !(flf = vl.vr_filocks))
+	||  !(flf = (KA_T)vl.vr_filocks))
 	    return(' ');
 #endif	/* PTXV<410 */
 
 	flp = flf;
 	do {
-	    if (kread((KA_T)flp, (char *)&f, sizeof(f)))
+	    if (kread(flp, (char *)&f, sizeof(f)))
 		break;
 	    if (f.set.l_sysid || f.set.l_pid != (pid_t)Lp->pid)
 		continue;
@@ -218,7 +223,7 @@ isvlocked(va)
 		return('u');
 	    }
 	    return(' ');
-	} while ((flp = f.next) && flp != flf);
+	} while ((flp = (KA_T)f.next) && flp != flf);
 	return(' ');
 }
 
@@ -229,32 +234,32 @@ isvlocked(va)
 
 void
 process_node(va)
-	caddr_t va;			/* vnode kernel space address */
+	KA_T va;			/* kernel vnode address */
 {
-	char *cp, dev_ch[32];
-	static unsigned long cvops;
+	char *cp, tbuf[32];
+	static KA_T cvops = (KA_T)0;
 	dev_t dev = 0;
 	int dl, fp, j, k, l, lp, pt, px;
 	struct l_dev *dp;
 	struct fifonode f;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	static int ft = 1;
-	static unsigned long fvops;
+	static KA_T fvops = (KA_T)0;
 	struct inode i;
-	KA_T ka, qp;
-	struct in_addr *la = (struct in_addr *)NULL;
+	KA_T ka, p, qp, sop;
+	unsigned char *la = (unsigned char *)NULL;
 	struct module_info mi;
-	static unsigned long nfsvops;
-	static unsigned long nmvops;
-	struct inpcb *p, pcb;
+	static KA_T nfsvops = (KA_T)0;
+	static KA_T nmvops = (KA_T)0;
+	struct inpcb pcb;
 	struct queue q;
 	struct qinit qi;
-	static unsigned long sckvops;
-	static unsigned long spvops;
-	struct so_so so, *sop;
+	static KA_T sckvops = (KA_T)0;
+	struct so_so so;
+	static KA_T spvops = (KA_T)0;
 	struct sockaddr_un ua;
-	static unsigned long uvops;
-	static unsigned long vvops;
+	static KA_T uvops = (KA_T)0;
+	static KA_T vvops = (KA_T)0;
 
 #if	defined(HAS_NFS)
 	struct rnode r;
@@ -262,9 +267,10 @@ process_node(va)
 
 	struct snode s;
 	KA_T sa;
-	struct stdata sd, *sp;
+	struct stdata sd;
 	int sin = 0;
 	int sns = 0;
+	KA_T sp;
 	int srn = 0;
 	int srv = 0;
 	char *ty;
@@ -279,19 +285,16 @@ process_node(va)
 	int nmst = 1;
 #endif	/* PTXV>=400 */
 
-#if	PTXV>=413
+#if	TCPV>=420
+	struct datab db;
 	struct domain dm;
+	struct msgb mb;
 	struct socket_peer peer;
 	struct protosw pr;
 	struct socket sck;
-#endif	/* PTXV>=413 */
-
-#if	PTXV>=421
-	struct datab db;
-	struct msgb mb;
 	struct unpcb ub;
 	struct sockaddr_un un;
-#endif	/* PTXV>=421 */
+#endif	/* TCPV>=420 */
 
 #if	defined(HASVXFS)
 	int svx = 0;
@@ -302,21 +305,21 @@ process_node(va)
  */
 	if (ft) {
 	    if (get_Nl_value("cvops", Drive_Nl, &cvops) < 0 || !cvops)
-		cvops = (unsigned long)0;
+		cvops = (KA_T)0;
 	    if (get_Nl_value("fvops", Drive_Nl, &fvops) < 0 || !fvops)
-		fvops = (unsigned long)0;
+		fvops = (KA_T)0;
 	    if (get_Nl_value("nfsvops", Drive_Nl, &nfsvops) < 0 || !nfsvops)
-		nfsvops = (unsigned long)0;
+		nfsvops = (KA_T)0;
 	    if (get_Nl_value("nmvops", Drive_Nl, &nmvops) < 0 || !nmvops)
-		nmvops = (unsigned long)0;
+		nmvops = (KA_T)0;
 	    if (get_Nl_value("sckvops", Drive_Nl, &sckvops) < 0 || !sckvops)
-		sckvops = (unsigned long)0;
+		sckvops = (KA_T)0;
 	    if (get_Nl_value("spvops", Drive_Nl, &spvops) < 0 || !spvops)
-		spvops = (unsigned long)0;
+		spvops = (KA_T)0;
 	    if (get_Nl_value("uvops", Drive_Nl, &uvops) < 0 || !uvops)
-		uvops = (unsigned long)0;
+		uvops = (KA_T)0;
 	    if (get_Nl_value("vvops", Drive_Nl, &vvops) < 0 || !vvops)
-		vvops = (unsigned long)0;
+		vvops = (KA_T)0;
 	    ft = 0;
 	}
 /*
@@ -325,71 +328,80 @@ process_node(va)
 
 read_vnode:
 
-	if ( ! va) {
+	if (!va) {
 	    enter_nm("no vnode address");
 	    return;
 	}
-	if (readvnode((caddr_t)va, &v)) {
-                   enter_nm(Namech);
-                   return;
-        }
+	if (readvnode(va, &v)) {
+	    enter_nm(Namech);
+	   return;
+	}
 
-# if	defined(HASNCACHE)
-	Lf->na = (unsigned long)va;
-# endif	/* defined(HASNCACHE) */
+#if	defined(HASNCACHE)
+	Lf->na = va;
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	if (!Fxopt) {
+	    Lf->fna = va;
+	    Lf->fsv |= FSV_NI;
+	}
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Determine the vnode type.
  */
-	if (uvops && uvops == (unsigned long)v.v_op)
+	if (uvops && uvops == (KA_T)v.v_op)
 	    Ntype = N_REGLR;
-	else if (fvops && fvops == (unsigned long)v.v_op)
+	else if (fvops && fvops == (KA_T)v.v_op)
 	    Ntype = N_FIFO;
 
 #if	defined(HAS_NFS)
-	else if (nfsvops && nfsvops == (unsigned long)v.v_op)
+	else if (nfsvops && nfsvops == (KA_T)v.v_op)
 	    Ntype = N_NFS;
 #endif	/* defined(HAS_NFS) */
 
-#if	PTXV>=413
-	else if (sckvops && sckvops == (unsigned long)v.v_op)
+#if	TCPV>=420
+	else if (sckvops && sckvops == (KA_T)v.v_op)
 	    Ntype = N_SOCK;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
-	else if (spvops && spvops == (unsigned long)v.v_op)
+	else if (spvops && spvops == (KA_T)v.v_op)
 	    Ntype = N_SPEC;
 
 #if	defined(HAS_CDFS)
-	else if (cvops && cvops == (unsigned long)v.v_op)
+	else if (cvops && cvops == (KA_T)v.v_op)
 	    Ntype = N_CDFS;
 #endif	/* defined(HAS_CDFS) */
 
 #if	defined(HASVXFS)
-	else if (vvops && vvops == (unsigned long)v.v_op)
+	else if (vvops && vvops == (KA_T)v.v_op)
 	    Ntype = N_VXFS;
 #endif	/* defined(HASVXFS) */
 
 #if	PTXV>=400
-	else if (nmvops && nmvops == (unsigned long)v.v_op) {
+	else if (nmvops && nmvops == (KA_T)v.v_op) {
 	    Ntype = N_NM;
 	/*
 	 * Read the namenode and enter the fattach addresses.
 	 */
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&n, sizeof(n))) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read namenode (%#x)", va, v.v_data);
+		    "vnode at %s: can't read namenode (%s)",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    if (n.nm_mountpt)
-		ent_fa(&va, (caddr_t *)&n.nm_mountpt, "->");
+		ent_fa(&va, (KA_T *)&n.nm_mountpt, "->");
 	/*
 	 * If this is the first time a namenode has been examined, and
 	 * if it has an nm_filevp pointer, reset the vnode address to
 	 * it and start over.
 	 */
 	    if (nmst && n.nm_filevp) {
-		va = (caddr_t)n.nm_filevp;
+		va = (KA_T)n.nm_filevp;
 		nmst = 0;
 		goto read_vnode;
 	    }
@@ -401,8 +413,17 @@ read_vnode:
 	/*
 	 * The file system type is unknown.
 	 */
+
+#if	PTXV>=450
+	    if ((v.v_flag & VSVTX) && !strcmp(Lf->fd, "txt")) {
+		Lf->sf = 0;
+		return;
+	    }
+#endif	/* PTXV>=450 */
+
 	    (void) sprintf(Namech,
-		"unknown file system type, v_op: %#x", v.v_op);
+		"unknown file system type, v_op: %s",
+		print_kptr((KA_T)v.v_op, (char *)NULL));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -414,9 +435,9 @@ read_vnode:
  * fattach addresses.
  */
 	if (v.v_vfsmountedhere
-	&&  kread((KA_T)v.v_vfsmountedhere, (char *)&favfs, sizeof(favfs)) == 0
+	&&  !kread((KA_T)v.v_vfsmountedhere, (char *)&favfs, sizeof(favfs))
 	&&  favfs.vfs_data)
-	    ent_fa(&va, &favfs.vfs_data, "<-");
+	    ent_fa(&va, (KA_T *)&favfs.vfs_data, "<-");
 #endif	/* PTXV>=400 */
 
 /*
@@ -426,12 +447,14 @@ read_vnode:
 /*
  * Establish the local virtual file system structure.
  */
-	if (v.v_vfsp == NULL)
-	    lvfs = NULL;
+	if (!v.v_vfsp)
+	    lvfs = (struct l_vfs *)NULL;
 	else {
-	    if ((lvfs = readvfs(&v)) == NULL) {
-                (void) sprintf(Namech, "bad vfs for %#x at %#x", va, v.v_vfsp);
-                enter_nm(Namech);
+	    if (!(lvfs = readvfs(&v))) {
+		(void) sprintf(Namech, "bad vfs for %s at %s",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_vfsp, (char *)NULL));
+		enter_nm(Namech);
 		return;
 	    }
 	}
@@ -446,7 +469,9 @@ read_vnode:
 		return;
 	    if (!v.v_data || read_cdnode(&v, lvfs, &dev)) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read cd_inode (%#x)", va, v.v_data);
+		    "vnode at %s: can't read cd_inode (%s)",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
@@ -459,16 +484,22 @@ read_vnode:
 		return;
 	    if (!v.v_data || read_vxinode(&v, lvfs, &dev)) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read vx_inode (%#x)", va, v.v_data);
+		    "vnode at %s: can't read vx_inode (%s)",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 
 # if	defined(HASNCACHE) && defined(HASVXFSDNLC)
 	    Lf->is_vxfs = 1;
-	    Lf->na = (unsigned long)v.v_data;
+	    Lf->na = (KA_T)v.v_data;
 # endif	/* defined(HASNCACHE) && defined(HASVXFSDNLC) */
 
+#if	defined(HASFSTRUCT)
+	    Lf->fna = (KA_T)v.v_data;
+#endif	/* defined(HASFSTRUCT) */
+
 	    break;
 #endif	/* defined(HASVXFS) */
 
@@ -477,33 +508,35 @@ read_vnode:
 		return;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&f, sizeof(f))) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read fifonode (%#x)", va, v.v_data);
+		    "vnode at %s: can't read fifonode (%s)",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    if (f.fn_snode.s_realvp
-	    &&  readvnode((caddr_t)f.fn_snode.s_realvp, &rv) == 0
+	    &&  !readvnode((KA_T)f.fn_snode.s_realvp, &rv)
 	    &&  rv.v_data) {
 		srv = 1;
 
 #if	defined(HAS_NFS)
-		if (nfsvops && nfsvops == (unsigned long)rv.v_op
-		&& readrnode((caddr_t)rv.v_data, &r) == 0) {
+		if (nfsvops && nfsvops == (KA_T)rv.v_op
+		&& readrnode((KA_T)rv.v_data, &r) == 0) {
 		    srn = 1;
 		    break;
 		}
 #endif	/* defined(HAS_NFS) */
 
 #if	defined(HASVXFS)
-		if (vvops && vvops == (unsigned long)rv.v_op
+		if (vvops && vvops == (KA_T)rv.v_op
 		&&  read_vxinode(&rv, lvfs, &dev) == 0) {
 		    svx = 1;
 		    break;
 		}
 #endif	defined(HASVXFS) */
 
-		if (uvops && uvops == (unsigned long)rv.v_op
-		&&  kread((KA_T)rv.v_data, (char *)&i, sizeof(i)) == 0) {
+		if (uvops && uvops == (KA_T)rv.v_op
+		&&  !kread((KA_T)rv.v_data, (char *)&i, sizeof(i))) {
 		    sin = 1;
 		    break;
 		}
@@ -514,9 +547,11 @@ read_vnode:
 	case N_NFS:
 	    if (Selinet)
 		return;
-	    if (!v.v_data || readrnode((caddr_t)v.v_data, &r)) {
+	    if (!v.v_data || readrnode((KA_T)v.v_data, &r)) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read rnode (%#x)", va, v.v_data);
+		    "vnode at %s: can't read rnode (%s)",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
@@ -530,14 +565,16 @@ read_vnode:
 		return;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&n, sizeof(n))) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read namenode (%#x)", va, v.v_data);
+		    "vnode at %s: can't read namenode (%s)",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	    }
 	    break;
 #endif	/* PTXV>=400 */
 
-#if	PTXV>=413
+#if	TCPV>=420
 	case N_SOCK:
 	    if (!v.v_snode && !v.v_stream) {
 
@@ -555,8 +592,6 @@ read_vnode:
 		if (!pr.pr_domain
 		||  kread((KA_T)pr.pr_domain, (char *)&dm, sizeof(dm)))
 		    break;
-
-#if	PTXV>=421
 		if (dm.dom_family == AF_UNIX) {
 
 		/*
@@ -569,6 +604,7 @@ read_vnode:
 		/*
 		 * Read sockaddr_un.
 		 */
+		    Namech[0] = '\0';
 		    if (!kread((KA_T)ub.unp_addr, (char *)&mb, sizeof(mb))
 		    &&  mb.b_datap
 		    &&  !kread((KA_T)mb.b_datap, (char *)&db, sizeof(db))
@@ -578,47 +614,59 @@ read_vnode:
 		    &&  un.sun_path)
 		    {
 
+# if	TCPV>=450
+
+		    /*
+		     * Enter path from variable length sockaddr_un.
+		     */
+			(void) read_vl_un((KA_T)db.db_base, (int)un.sun_len);
+# else	/* TCPV<450 */
+
 		    /*
 		     * Enter path from sockaddr_un.
 		     */
 			un.sun_path[sizeof(un.sun_path) - 1] - '\0';
-			if ((l = strlen(un.sun_path)) >= (MAXPATHLEN - 1))
-			    l = MAXPATHLEN - 1;
+			if ((l = strlen(un.sun_path)) >= (MAXPATHLEN - 2))
+			    l = MAXPATHLEN - 2;
 			(void) strncpy(Namech, un.sun_path, l);
 			Namech[l] = '\0';
+# endif	/* TCPV>=450 */
+
 		    }
+
 		    if (ub.unp_vnode) {
 			
-		     /*
-		      * If the socket is associated with a vnode, try to
-		      * interpret it instead, possibly using only the name
-		      * from the sockaddr_un of this VSOCK vnode.
-		      */
+		    /*
+		     * If the socket is associated with a vnode, try to
+		     * interpret it instead, possibly using only the name
+		     * from the sockaddr_un of this VSOCK vnode.
+		     */
 			if (Funix)
 			    Lf->sf |= SELUNX;
 			usst = 1;
-			va = (caddr_t)ub.unp_vnode;
+			va = (KA_T)ub.unp_vnode;
 			goto read_vnode;
-		     }
+		    } else {
+			if (Namech[0] && is_file_named(Namech, 0))
+			    Lf->sf |= SELNM;
+		    }
 		    (void) process_unix_socket(&sck, &ub);
 		    return;
-	        }
-# endif	/* PTXV>=421 */
-
+		}
 		if (dm.dom_family != AF_INET)
 		    break;
 		switch (pr.pr_protocol) {
 		case IPPROTO_ICMP:
 		    (void) process_socket("ICMP", AF_INET, IPPROTO_ICMP,
-			(KA_T)sck.so_pcb);
+			(KA_T)sck.so_pcb, (void *)&sck);
 		    return;
 		case IPPROTO_TCP:
 		    (void) process_socket("TCP", AF_INET, IPPROTO_TCP,
-			(KA_T)sck.so_pcb);
+			(KA_T)sck.so_pcb, (void *)&sck);
 		    return;
 		case IPPROTO_UDP:
 		    (void) process_socket("UDP", AF_INET, IPPROTO_UDP,
-			(KA_T)sck.so_pcb);
+			(KA_T)sck.so_pcb, (void *)&sck);
 		    return;
 		default:
 		    (void) printiproto(pr.pr_protocol);
@@ -628,14 +676,14 @@ read_vnode:
 	    }
 	    /* Change node type to N_SPEC and fall through. */
 	    Ntype = N_SPEC;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	case N_SPEC:
-	    p = (struct inpcb *)NULL;
+	    p = (KA_T)NULL;
 	    pt = -1;
-	    sop = (struct so_so *)NULL;
-	    if ((sa = (KA_T)v.v_snode) == NULL
-	    &&  (sa = (KA_T)v.v_data) == NULL) {
+	    sop = (KA_T)NULL;
+	    if (!(sa = (KA_T)v.v_snode)
+	    &&  !(sa = (KA_T)v.v_data)) {
 		sns = 0;
 		break;
 	    }
@@ -644,7 +692,8 @@ read_vnode:
 	 */
 	    if (kread(sa, (char *)&s, sizeof(s))) {
 		(void) sprintf(Namech,
-		    "vnode at %#x: can't read snode (%#x)", va, sa);
+		    "vnode at %s: can't read snode (%s)",
+		    print_kptr(va, tbuf), print_kptr(sa, (char *)NULL));
 		enter_nm(Namech);
 		return;
 	   }
@@ -654,31 +703,33 @@ read_vnode:
 	   /*
 	    * Read the "real" vnode, if there is one, then read its inode.
 	    */
-		if (readvnode((caddr_t)s.s_realvp, &rv)) {
+		if (readvnode((KA_T)s.s_realvp, &rv)) {
 		    (void) sprintf(Namech,
-			"snode at %#x; can't read real vnode at %#x",
-			sa, s.s_realvp);
+			"snode at %s; can't read real vnode at %s",
+			print_kptr(sa, tbuf),
+			print_kptr((KA_T)s.s_realvp, (char *)NULL));
 		    return;
 		}
 		srv = 1;
 		if (rv.v_data) {
-		    if (kread((KA_T)v.v_data, (char *)&i, sizeof(i)) == 0) {
+		    if (!kread((KA_T)rv.v_data, (char *)&i, sizeof(i))) {
 			sin = 1;
 
-#if	defined(HASLFILEADD)
-			Lf->ina = rv.v_data;
-			Lf->ina_def = 1;
-#endif	/* defined(HASLFILEADD) */
+#if	defined(HASFSTRUCT)
+			if (Fxopt) {
+			    Lf->fna = (KA_T)rv.v_data;
+			    Lf->fsv |= FSV_NI;
+			}
+#endif	/* defined(HASFSTRUCT) */
 
-		
 		    }
 		}
 	    }
 
 #if	PTXV<400
-	    if ((sp = s.s_sptr) == NULL)
+	    if (!(sp = (KA_T)s.s_sptr))
 #else	/* PTXV>=400 */
-	    if ((sp = v.v_stream) == NULL)
+	    if (!(sp = (KA_T)v.v_stream))
 #endif	/*PTXV<400 */
 		break;
 	/*
@@ -708,55 +759,70 @@ read_vnode:
 	 * the private pointer of its queue structure as a possible
 	 * struct so_so address.
 	 */
-	    if (kread((KA_T)sp, (char *)&sd, sizeof(sd)) == 0) {
-		dl = sizeof(dev_ch) - 1;
-		dev_ch[dl] = '\0';
+	    if (kread(sp, (char *)&sd, sizeof(sd)) == 0) {
+		dl = sizeof(tbuf) - 1;
+		tbuf[dl] = '\0';
 		qp = (KA_T)sd.sd_wrq;
 		for (j = 0; qp && j < 20; j++, qp = (KA_T)q.q_next) {
 		    if (kread(qp, (char *)&q, sizeof(q)))
 			break;
-		    if ((ka = (KA_T)q.q_qinfo) == (KA_T)NULL
+		    if (!(ka = (KA_T)q.q_qinfo)
 		    ||  kread(ka, (char *)&qi, sizeof(qi)))
 			continue;
-		    if ((ka = (KA_T)qi.qi_minfo) == (KA_T)NULL
+		    if (!(ka = (KA_T)qi.qi_minfo)
 		    ||  kread(ka, (char *)&mi, sizeof(mi)))
 			continue;
-		    if ((ka = (KA_T)mi.mi_idname) == (KA_T)NULL
-		    ||  kread(ka, dev_ch, dl))
+		    if (!(ka = (KA_T)mi.mi_idname) || kread(ka, tbuf, dl))
 			continue;
-		    if ((l = strlen(dev_ch)) < 1)
+		    if ((l = strlen(tbuf)) < 1)
 			continue;
-		    if (l >= 4 && strcasecmp(&dev_ch[l - 4], "head") == 0)
+		    if (l >= 4 && strcasecmp(&tbuf[l - 4], "head") == 0)
 			continue;
-		    if (q.q_next == NULL && q.q_ptr != NULL
-		    &&  (px = isip(dev_ch)) >= 0
+		    if (!q.q_next && q.q_ptr && (px = isip(tbuf)) >= 0
 
-#if	PTXV>=413
+#if	TCPV>=420
 		    &&  !kread((KA_T)q.q_ptr, (char *)&peer, sizeof(peer))
+# if	TCPV>=450
+		    &&  peer.sop_socket
+		    &&  !kread((KA_T)peer.sop_socket, (char *)&sck, sizeof(sck))
+		    &&  sck.so_pcb
+# else	/* TCPV<450 */
 		    &&  peer.sop_inpcb
-#endif	/* PTXV>=413 */
+# endif	/* TCPV>=450 */
+#endif	/* TCPV>=420 */
 
 		    ) {
 			process_socket(IPtab[px].nm, IPtab[px].fam,
 				       IPtab[px].ipid,
 
-#if	PTXV>=413
-				       (KA_T)peer.sop_inpcb
-#else	/* PTXV<413 */
+#if	TCPV>=420
+# if	TCPV>=450
+				       (KA_T)sck.so_pcb,
+# else	/* TCPV<450 */
+				       (KA_T)peer.sop_inpcb,
+# endif	/* TCPV>=450 */
+#else	/* TCPV<420 */
+				       (KA_T)q.q_ptr,
+#endif	/* TCPV>=420 */
 
-				       (KA_T)q.q_ptr
-#endif	/* PTXV>=413 */
-
-		        );
+				       (void *)NULL
+			);
 			    return;
 		    }
-		    if (cp && strcmp(cp, dev_ch) == 0) {
+		    if (cp && strcmp(cp, tbuf) == 0) {
 			if (q.q_ptr && pt < 0 && (px = isip(cp)) >= 0
 
-#if	PTXV>=413
-			 && !kread((KA_T)q.q_ptr, (char *)&peer, sizeof(peer))
+#if	TCPV>=420
+			&&  !kread((KA_T)q.q_ptr, (char *)&peer, sizeof(peer))
+# if	TCPV>=450
+			&&  peer.sop_socket
+			&&  !kread((KA_T)peer.sop_socket, (char *)&sck,
+				   sizeof(sck))
+			&&  sck.so_pcb
+# else	/* TCPV<450 */
 			 &&  peer.sop_inpcb
-#endif	/* PTXV>=413 */
+# endif	/* TCPV>=450 */
+#endif	/* TCPV>=420 */
 
 			) {
 
@@ -774,20 +840,23 @@ read_vnode:
 			    }
 			    if (pt >= 0) {
 
-#if	PTXV>=413
-				p = peer.sop_inpcb;
-#else	/* PTXV<413 */
-				p = (struct inpcb *)q.q_ptr;
-#endif	/* PTXV>=413 */
+#if	TCPV>=420
+# if	TCPV>=450
+				p = (KA_T)sck.so_pcb;
+# else	/* TCPV<450 */
+				p = (KA_T)peer.sop_inpcb;
+# endif	/* TCPV>=450 */
+#else	/* TCPV<420 */
+				p = (KA_T)q.q_ptr;
+#endif	/* TCPV>=420 */
 
 				(void) strcpy(Lf->iproto, IPtab[px].nm);
 			    }
-		        }
+			}
 			continue;
 		    }
-		    if (q.q_ptr && strcasecmp(dev_ch, "sockmod") == 0
-		    &&  sop == (struct so_so *)NULL)
-			sop = (struct so_so *)q.q_ptr;
+		    if (q.q_ptr && strcasecmp(tbuf, "sockmod") == 0 && !sop)
+			sop = (KA_T)q.q_ptr;
 		    if (k) {
 			if ((k + 2) > (MAXPATHLEN - 1))
 			    break;
@@ -796,19 +865,19 @@ read_vnode:
 		    }
 		    if ((k + l) > (MAXPATHLEN - 1))
 			break;
-		    (void) strcpy(&Namech[k], dev_ch);
+		    (void) strcpy(&Namech[k], tbuf);
 		    k += l;
 		}
 	    }
 	    if (p && pt >= 0) {
-		sop = NULL;
+		sop = (KA_T)NULL;
 
 	/*
 	 * If the stream has a TCP or UDP module with a PCB
 	 * pointer, print any associated local and foreign
 	 * Internet addresses.
 	 */
-		if (kread((KA_T)p, (char *)&pcb, sizeof(pcb)))
+		if (kread(p, (char *)&pcb, sizeof(pcb)))
 		    break;
 		if (Fnet)
 		    Lf->sf |= SELNET;
@@ -820,7 +889,7 @@ read_vnode:
 	    /*
 	     * Print the local and foreign addresses from the PCB.
 	     */
-		la = (struct in_addr *)&pcb.inp_laddr;
+		la = (unsigned char *)&pcb.inp_laddr;
 		lp = (int)ntohs(pcb.inp_lport);
 
 #if	defined(HASINADDRSTR)
@@ -830,43 +899,47 @@ read_vnode:
 #endif	/* defined(HASINADDRSTR) */
 
 		{
-		    fa = (struct in_addr *)&pcb.inp_faddr;
+		    fa = (unsigned char *)&pcb.inp_faddr;
 		    fp = (int)ntohs(pcb.inp_fport);
 		}
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
 		Lf->inp_ty = 2;
-	    } else if (sop != (struct so_so *)NULL) {
+	    } else if (sop) {
 		    
 	    /*
 	     * The stream has a "sockmod" member and its queue structure
 	     * has a private pointer; see if it points to Unix domain
 	     * socket information; if it does, read its so_so structure.
 	     */
-		if (kread((KA_T)sop, (char *)&so, sizeof(so)))
-		    sop = (struct so_so *)NULL;
+		if (kread(sop, (char *)&so, sizeof(so)))
+		    sop = (KA_T)NULL;
 	    }
 	    break;
 	case N_REGLR:
 	default:
 	    if (Selinet)
-			return;
+		return;
 	    if (v.v_type != VBLK) {
 		if (!v.v_data
 		||  kread((KA_T)v.v_data, (char *)&i, sizeof(i))) {
 		    (void) sprintf(Namech,
-			"vnode at %#x: can't read inode (%#x)", va, v.v_data);
+			"vnode at %s: can't read inode (%s)",
+			print_kptr(va, tbuf),
+			print_kptr((KA_T)v.v_data, (char *)NULL));
 		    enter_nm(Namech);
 		    return;
 		}
 		sin = 1;
+
+#if	defined(HASFSTRUCT)
+		if (Fxopt) {
+		    Lf->fna = (KA_T)v.v_data;
+		    Lf->fsv |= FSV_NI;
+		}
+#endif	/* defined(HASFSTRUCT) */
+
 	    }
-
-#if	defined(HASLFILEADD)
-	    Lf->ina = v.v_data;
-	    Lf->ina_def = 1;
-#endif	/* defined(HASLFILEADD) */
-
 	}
 	if (Selinet)
 	    return;
@@ -917,10 +990,8 @@ read_vnode:
 		Lf->dev_def = 1;	/* dev is set in read_vxnode() */
 #endif	/* defined(HASVXFS) */
 
-	    else {
-		(void) sprintf(dev_ch, "0x%08x", va);
-		enter_dev_ch(dev_ch);
-	    }
+	    else
+		enter_dev_ch(print_kptr((KA_T)va, (char *)NULL));
 	    break;
 
 #if	defined(HASVXFS)
@@ -929,19 +1000,19 @@ read_vnode:
 	    break;
 #endif	/* defined(HASVXFS) */
 
-#if	PTXV>=413
+#if	TCPV>=420
 	case N_SOCK:
 	    if (v.v_rdev == NODEV)
-		enter_dev_ch("     NODEV");
+		enter_dev_ch("NODEV");
 	    else {
 		dev = v.v_rdev;
 		Lf->dev_def = 1;
 	    }
 	    break;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	case N_SPEC:
-	    if (sop != (struct so_so *)NULL)
+	    if (sop)
 		dev = so.lux_dev.addr.tu_addr.dev;
 	    else
 		dev = sns ? s.s_dev : v.v_rdev;
@@ -955,9 +1026,8 @@ read_vnode:
 		if (srv) {
 		    dev = rv.v_rdev;
 		    Lf->dev_def = 1;
-		} else if (i.i_devvp != NULL
-		       &&  kread((KA_T)i.i_devvp, (char *)&uv, sizeof(uv))
-		       == 0)
+		} else if (i.i_devvp
+		       &&  !kread((KA_T)i.i_devvp, (char *)&uv, sizeof(uv)))
 		{
 		    dev = uv.v_rdev;
 		    Lf->dev_def = 1;
@@ -981,7 +1051,6 @@ read_vnode:
 	    break;
 #endif	/* defined(HASVXFS) */
 
-
 	case N_FIFO:
 	    if (sin) {
 		Lf->inode = (unsigned long)i.i_number;
@@ -1023,7 +1092,7 @@ read_vnode:
 	     }
 	     break;
 
-#if	PTXV>=413
+#if	TCPV>=420
 	case N_SOCK:
 	    /* The sock_vnodeops type vnode has no inode number. */
 	    if (!Lf->iproto[0]) {
@@ -1031,7 +1100,7 @@ read_vnode:
 		Lf->inp_ty = 2;
 	    }
 	    break;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	case N_SPEC:
 	    if (sop) {
@@ -1060,7 +1129,6 @@ read_vnode:
 		break;
 #endif	/* defined(HASVXFS) */
 
-
 	     case N_FIFO:
 
 #if	defined(HASVXFS)
@@ -1073,7 +1141,7 @@ read_vnode:
 		if (Fsize || (Lf->access != 'r' && Lf->access != 'w')) {
 		    (void) sprintf(Namech, "rd=%#x; wr=%#x",
 			f.fn_rptr, f.fn_wptr);
-		    Lf->sz = (unsigned long)f.fn_size;
+		    Lf->sz = (SZOFFTYPE)f.fn_size;
 		    Lf->sz_def = 1;
 		    break;
 		}
@@ -1089,14 +1157,14 @@ read_vnode:
 
 #if	defined(HAS_NFS)
 	     case N_NFS:
-		Lf->sz = (unsigned long)r.r_size;
+		Lf->sz = (SZOFFTYPE)r.r_size;
 		Lf->sz_def = 1;
 		break;
 #endif	/* defined(HAS_NFS) */
 
 #if	PTXV>=400
 	     case N_NM:
-		Lf->sz = (unsigned long)n.nm_vattr.va_size;
+		Lf->sz = (SZOFFTYPE)n.nm_vattr.va_size;
 		Lf->sz_def = 1;
 		break;
 #endif	/* PTXV>=400 */
@@ -1104,7 +1172,7 @@ read_vnode:
 	     case N_REGLR:
 		if (type == VREG || type == VDIR || (type == VSOCK && usst)) {
 		    if (sin) {
-			Lf->sz = (unsigned long)
+			Lf->sz = (SZOFFTYPE)
 
 #if	PTXV<420
 				  i.i_ic.ic_size.val[0];
@@ -1118,11 +1186,11 @@ read_vnode:
 		    Lf->off_def = 1;
 		break;
 
-#if	PTXV>=413
+#if	TCPV>=420
 	     case N_SOCK:
 		Lf->off_def = 1;
 		break;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	     case N_SPEC:
 		if (type == VCHR && !Fsize)
@@ -1130,6 +1198,64 @@ read_vnode:
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+
+#if	defined(HAS_CDFS)
+	    case N_CDFS:
+		/* Lf->nlink and Lf->nlink_def are set in read_cdnode() */
+	    break;
+#endif	/* defined(HAS_CDFS) */
+
+#if	defined(HASVXFS)
+	    case N_VXFS:
+		/* Lf->nlink and Lf->nlink_def are set in read_vxnode() */
+	    break;
+#endif	/* defined(HASVXFS) */
+
+	    case N_FIFO:
+		if (sin) {
+		    Lf->nlink = (long)i.i_nlink;
+		    Lf->nlink_def = 1;
+		}
+
+#if	defined(HAS_NFS)
+		else if (srn) {
+		    Lf->nlink = (long)r.r_nfsattr.na_nlink;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* defined(HAS_NFS) */
+
+		break;
+
+#if	defined(HAS_NFS)
+	    case N_NFS:
+		Lf->nlink = (long)r.r_nfsattr.na_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(HAS_NFS) */
+
+#if	PTXV>=400
+	    case N_NM:
+		Lf->nlink = (long)n.nm_vattr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* PTXV>=400 */
+
+	    case N_SPEC:
+	    case N_REGLR:
+		if (sin) {
+		    Lf->nlink = (long)i.i_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 
 #if	defined(HAS_NFS)
 /*
@@ -1224,31 +1350,47 @@ read_vnode:
  * address there.
  */
 	if (Ntype == N_FIFO && f.fn_mate && Namech[0] == '\0' && !sin && !srn)
-	    (void) sprintf(Namech, "->0x%08x", f.fn_mate);
+	    (void) sprintf(Namech, "->%s",
+		print_kptr((KA_T)f.fn_mate, (char *)NULL));
 #endif	/* PTXV>=400 */
 
 /*
  * If this is a "sockmod" special file and it has a so_so structure,
  * try to retrieve the Unix domain socket path.
  */
-	if (Ntype == N_SPEC && sop != (struct so_so *)NULL) {
+	if (Ntype == N_SPEC && sop) {
 	    if (Funix)
 		Lf->sf |= SELUNX;
 	    (void) strcpy(Lf->type, "unix");
 	    if ((Namech[0] == '\0' || !strncmp(Namech, "STR:", strlen("STR:")))
-	    &&  so.laddr.buf != (char *)NULL
-	    &&  so.laddr.len == sizeof(ua))
+	    &&  so.laddr.buf)
 	    {
-		if (kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua)) == 0) {
+
+#if	TCPV>=450
+		if (!kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))
+		&&  ua.sun_family == AF_UNIX
+		&&  read_vl_un((KA_T)so.laddr.buf, (int)ua.sun_len))
+		{
+		    if (is_file_named(Namech, 0))
+			Lf->sf |= SELNM;
+		}
+#else	/* TCPV<450 */
+		if (so.laddr.len = sizeof(ua)
+		&& !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua)))
+		{
 		    ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
 		    (void) strcpy(Namech, ua.sun_path);
+		    if (is_file_named(Namech, 0))
+			Lf->sf |= SELNM;
 		}
+#endif	/* TCPV>=450 */
+
 	    }
 	}
 /*
  * Test for specified file.
  */
-	if (Sfile && is_file_named(NULL, type))
+	if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
 	    Lf->sf |= SELNM;
 /*
  * Enter name characters.
@@ -1256,3 +1398,52 @@ read_vnode:
 	if (Namech[0])
 	    enter_nm(Namech);
 }
+
+
+#if	TCPV>=450
+/*
+ * read_vl_un() - read variable length Unix socket address name for TCP
+ *		  versions 4.5 and above
+ */
+
+static int
+read_vl_un(addr, len)
+	KA_T addr;			/* kernel address */
+	int len;			/* length */
+{
+	int nml;
+	static struct sockaddr_un *vu = (struct sockaddr_un *)NULL;
+	static MALLOC_S vul = (MALLOC_S)0;
+
+	nml = len - offsetof(struct sockaddr_un, sun_path);
+	if (nml <= 0)
+	    return(0);
+/*
+ * Allocate space.
+ */
+	if (len > (int)vul) {
+	    vul = (MALLOC_S)len;
+	    if (vu)
+		vu = (struct sockaddr_un *)realloc((MALLOC_P *)vu, vul);
+	    else
+		vu = (struct sockaddr_un *)malloc(vul);
+	    if (!vu) {
+		(void) fprintf(stderr, "%s: no space (%d) for sockaddr_un\n",
+		    Pn, len);
+		Exit(1);
+	    }
+	}
+/*
+ * Enter path from variable length sockaddr_un.
+ */
+	if (!kread(addr, (char *)vu, len)) {
+	    nml = len - offsetof(struct sockaddr_un, sun_path);
+	    if (nml > (MAXPATHLEN - 2))
+		nml = MAXPATHLEN - 2;
+	    (void) strncpy(Namech, vu->sun_path, nml);
+	    Namech[nml - 1] = '\0';
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* TCPV>=450 */
