@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.10 98/02/13 12:44:24 abe Exp $
+ * $Id: dlsof.h,v 1.25 2000/06/25 17:31:11 abe Exp $
  */
 
 
@@ -75,6 +75,11 @@
 #include <netinet/in.h>
 
 # if	defined(solaris)
+#  if	solaris>=70000
+#include <sys/conf.h>
+#include <sys/systeminfo.h>
+#  endif	/* solaris>=70000 */
+
 #define	_KERNEL
 #define	MI_HRTIMING
 #include <inet/led.h>
@@ -84,8 +89,17 @@
 #  endif	/* solaris<20600 */
 
 #include <inet/common.h>
+
+#  if	solaris>=70000
+#include <sys/stropts.h>
+#  endif	/* solaris>=70000 */
+
 #include <inet/mi.h>
 
+#  if	solaris>=80000
+#include <netinet/ip6.h>
+#  endif	/* solaris>=80000 */
+
 #include <inet/ip.h>
 #undef	_KERNEL
 #undef	MI_HRTIMING
@@ -97,7 +111,17 @@
 #undef	exit
 #undef	rval_t
 #undef	strsignal
+
+#  if	solaris>=80000
+#define	_KERNEL	1
+#  endif	/* solaris>=80000 */
+
 #include <inet/tcp.h>
+
+#  if	solaris>=80000
+#undef	_KERNEL
+#  endif	/* solaris>=80000 */
+
 # else	/* !solaris */
 #include <netinet/tcp.h>
 #include <netinet/tcp_fsm.h>
@@ -124,15 +148,15 @@
 #include <sys/fs/hsfs_node.h>
 #include <sys/fs/lofs_node.h>
 
-# if	solaris>=20600
+#  if	solaris>=20600
 #define	_KERNEL
-# endif	solaris>=20600
+#  endif	/* solaris>=20600 */
 
 #include <sys/fs/namenode.h>
 
-# if	solaris>=20600
+#  if	solaris>=20600
 #undef	_KERNEL
-# endif	solaris>=20600
+#  endif	/* solaris>=20600 */
 
 #include <sys/tihdr.h>
 
@@ -159,10 +183,11 @@
 #include <rpc/pmap_prot.h>
 
 /*
- * Dummy hsnode structure for SunOS 4.1.3 and below.  This should come from
+ * Dummy hsnode structure for below SunOS 4.1.3.  This should come from
  * hsfs_node.h, but it has unresolvable protoype declaration conflicts.
  */
 
+#  if	SUNOSV<40103
 struct  hsnode {
 	struct hsnode	*dummy1[3];
 	struct vnode	dummy2;
@@ -171,6 +196,21 @@ struct  hsnode {
 		u_int	ext_size;    	/* size */
 	} hs_dirent;
 };
+#  else	/* SUNOSV>=40103 */
+#include <hsfs/hsfs_spec.h>
+struct cred;
+#define	strcpy	LSOF_strcpy
+#define	strncpy	LSOF_strncpy
+
+#   if	defined(HAS_HSFS_NODE)
+#include <hsfs/hsfs_node.h>
+#   else	/* !defined(HAS_HSFS_NODE) */
+#include <hsfs/hsnode.h>
+#   endif	/* defined(HAS_HSFS_NODE) */
+
+#undef	strcpy
+#undef	strncpy
+#  endif	/* SUNOSV<40103 */
 # endif	/* solaris */
 
 # if	defined(solaris)
@@ -282,7 +322,7 @@ struct lock_descriptor {
 
 struct mvfsnode {
 	unsigned long d1[6];
-	unsigned long m_ino;			/* node number */
+	unsigned long m_ino;		/* node number */
 };
 
 extern int nlist();
@@ -292,10 +332,9 @@ extern char *sys_errlist[];
 #define	strerror(errno)	sys_errlist[errno]
 # endif
 
-# if	defined(HAS_AFS)
+# if	defined(HAS_AFS) && !defined(AFSAPATHDEF)
 #define	AFSAPATHDEF	"/usr/vice/etc/modload/libafs"
-#define	AFSDEV		1	/* AFS "fake" device number */
-# endif	/* defined(HAS_AFS) */
+# endif	/* defined(HAS_AFS) && !defined(AFSAPATHDEF) */
 
 # if	defined(NOCONST)
 #define	COMP_P		void
@@ -304,43 +343,72 @@ extern char *sys_errlist[];
 # endif
 
 #define	CWDLEN		(MAXPATHLEN+1)
-#define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		unsigned long
+#define DEVINCR		1024		/* device table malloc() increment */
+#define	DIRTYPE		dirent
+
+# if	defined(solaris)
+#  if	solaris>=70000
+typedef	uintptr_t	KA_T;
+#  else	/* solaris<70000 */
+typedef	void *		KA_T;
+#  endif	/* solaris>=70000 */
+# else	/* !defined(solaris) */
+typedef	unsigned long	KA_T;
+# endif	/* defined(solaris) */
+
+# if	defined(solaris) && solaris>=20501
+#define	KMEM		"/dev/mem"
+# else	/* !defined(solaris) || solaris<20501 */
 #define	KMEM		"/dev/kmem"
+# endif	/* defined(solaris) && solaris>=20501 */
+
 #define MALLOC_P	char
 #define FREE_P		MALLOC_P
-#define MAXSEGS		100	/* maximum text segments */
 #define MALLOC_S	unsigned
 
+# if	!defined(MAXEND)
+#define	MAXEND		0x7fffffff
+# endif	/* !defined(MAXEND) */
+
+#define MAXSEGS		100		/* maximum text segments */
+
 # if	defined(solaris)
 #define	DINAMEL		32
+
+#  if	solaris>=70000
+#define	KA_T_FMT_X	"0x%p"
+#  endif	/* solaris>=70000 */
+
 #define NETCLNML	8
 #define	N_UNIX		"/dev/ksyms"
-#define	PROCMIN		5	/* processes that make a "good" scan */
+#define	PROCMIN		5		/* processes that make a "good" scan */
 
 #  if	defined(HASPROCFS)
-#define	PR_ROOTINO	2	/* root inode for proc file system */
+#define	PR_ROOTINO	2		/* root inode for proc file system */
 #  endif	/* defined(HASPROCFS) */
 
 # else	/* !defined(solaris) */
-#define CLONEMAJ	37	/* major device number for clone device */
-#define	MEMMOVE		memcpy	/* for fill_portmap() in print.c */
+#define CLONEMAJ	37		/* major device number for clone
+					 * device */
+#define	MEMMOVE		memcpy		/* for fill_portmap() in print.c */
 #define N_UNIX		"/vmunix"
-#define	PROCMIN		3	/* processes that make a "good" scan */
+#define	PROCMIN		3		/* processes that make a "good" scan */
 # endif	/* defined(solaris) */
 
-#define	PROCDFLT	256	/* default size for local proc table */
+#define	PROCDFLT	256		/* default size for local proc table */
 #define PROCSIZE	sizeof(struct proc)
-#define	PROCTRYLM	5	/* times to try to read proc table */
+#define	PROCTRYLM	5		/* times to try to read proc table */
 #define QSORT_P		char
 #define	READLEN_T	int
 #define STRNCPY_L	int
-#define	STRNML		32	/* stream name length (maximum) */
+#define	STRNML		32		/* stream name length (maximum) */
 
-# if	defined(solaris) && solaris>=20600
+# if	defined(solaris) && solaris>=20501
 #define	SZOFFTYPE	unsigned long long
-				/* size and offset internal storage type */
-#define	SZOFFPSPEC	"ll"	/* SZOFFTYPE print specification modifier */
+					/* size and offset internal storage
+					 * type */
+#define	SZOFFPSPEC	"ll"		/* SZOFFTYPE print specification
+					 * modifier */
 # endif	/* defined(solaris) && solaris>=20600 */
 
 #define U_SIZE		sizeof(struct user)
@@ -353,20 +421,25 @@ extern char *sys_errlist[];
 # if	defined(HAS_AFS)
 
 #  if	defined(HASAOPT)
-extern char *AFSApath;		/* alternate AFS name list path (from -a) */
+extern char *AFSApath;			/* alternate AFS name list path
+					 * (from -a) */
 #  endif	/* defined(HASAOPT) */
 
+extern dev_t AFSdev;			/* AFS file system device number */
+extern int AFSdevStat;			/* AFS file system device number
+					 * status: 0 = unknown; 1 = known */
+
 #  if	defined(solaris)
-extern int AFSfstype;		/* AFS file system type index */
+extern int AFSfstype;			/* AFS file system type index */
 #  endif	/* defined(solaris) */
 
-extern struct vfs *AFSVfsp;	/* AFS struct vfs kernel pointer */
+extern KA_T AFSVfsp;			/* AFS struct vfs kernel pointer */
 # endif	/* defined(HAS_AFS) */
 
 struct clone {
-	struct l_dev cd;	/* device, inode, name, and verify */
-	int n;			/* network flag */
-	struct clone *next;	/* forward link */
+	struct l_dev cd;		/* device, inode, name, and verify */
+	int n;				/* network flag */
+	struct clone *next;		/* forward link */
 };
 extern struct clone *Clone;
 
@@ -384,7 +457,7 @@ extern int HaveCloneMaj;
 extern kvm_t *Kd;
 
 struct l_vfs {
-	struct vfs *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
 	dev_t dev;			/* device */
@@ -393,6 +466,11 @@ struct l_vfs {
 	ino_t fs_ino;			/* file system inode number */
 # endif	/* defined(HASFSINO) */
 
+# if	defined(solaris) && solaris>=80000
+	nlink_t nlink;			/* directory link count */
+	off_t size;			/* directory size */
+# endif	/* defined(solaris) && solaris>=80000 */
+
 	struct l_vfs *next;		/* forward link */
 };
 extern struct l_vfs *Lvfs;
@@ -412,7 +490,12 @@ struct mounts {
 
 # if	defined(HASFSTYPE)
 	char *fstype;			/* file system type */
-# endif
+# endif	/& defined(HASFSTYPE) */
+
+# if	defined(solaris) && solaris>=80000
+	nlink_t nlink;			/* directory st_nlink */
+	off_t size;			/* directory st_size */
+# endif	/* defined(solaris) && solaris>=80000 */
 
 };
 
@@ -442,6 +525,16 @@ struct sfile {
 };
 
 
+/*
+ * VxFS definitions
+ */
+
+#define	VXVOP_FDD	0		/* Vvops[] index for fdd_vnops */
+#define	VXVOP_FDDCH	1		/* Vvops[] index for fdd_chain_vnops */
+#define	VXVOP_REG	2		/* Vvops[] index for vx_vnodeops */
+#define	VXVOP_NUM	3		/* number of Vvops[] entries */
+
+
 /*
  * Kernel name list definitions
  */
@@ -478,6 +571,11 @@ struct sfile {
 
 #define	CVFS_DEVSAVE 	1
 
+# if	defined(solaris) && solaris>=80000
+#define	CVFS_NLKSAVE	1
+#define	CVFS_SZSAVE	1
+# endif	/* defined(solaris) && solaris>=80000 */
+
 
 /*
  * Definitions for rnch.c
@@ -485,6 +583,14 @@ struct sfile {
 
 # if	defined(HASNCACHE)
 #include <sys/dnlc.h>
+
+#  if	!defined(NC_NAMLEN)
+#define	HASDNLCPTR	1
+#  endif	/* !defined(NC_NAMLEN) */
+
+#  if	defined(solaris) && solaris>=80000
+#define	NCACHE_NEGVN	"negative_cache_vnode"
+#  endif	/* defined(solaris) && solaris>=80000 */
 # endif	/* defined(HASNCACHE) */
 
 #endif	/* SUN_LSOF_H */
