@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.5 97/10/24 07:55:47 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.12 2000/06/25 17:31:11 abe Exp $";
 #endif
 
 
@@ -48,9 +48,40 @@ static char *rcsid = "$Id: dnode2.c,v 1.5 97/10/24 07:55:47 abe Exp $";
 #include <sys/fs/vx_fs.h>
 # endif	/* defined(HASVXFS_FS_H) && !defined(HASVXFS_VX_INODE) */
 
-# if	defined(HASVXFS_SOL_H)
+# if	HASVXFS_SOL_H
 #include <sys/fs/vx_sol.h>
-# endif	/* HASVXFS_SOL_H */
+# endif	/* defined(HSVXFS_SOL_H) */
+
+# if	defined(HASVXFS_SOLARIS_H) && defined(HASVXFS_U64_T)
+#include <sys/fs/vx_solaris.h>
+# endif	/* defined(HASVXFS_SOLARIS_H) && defined(HASVXFS_U64_T) */
+
+# if	defined(HASVXFS_MACHDEP_H)
+#  if	defined(HASVXFS_OFF32_T) && solaris>=70000
+#define	off32_t	VXFS_off32_t
+#  endif	/* defined(HASVXFS_OFF32_T) && solaris>=70000 */
+#include <sys/fs/vx_machdep.h>
+# endif	/* defined(HASVXFS_MACHDEP_H) */
+
+# if	defined(HASVXFS_SOLARIS_H)
+struct kdm_vnode {			/* dummy for <sys/fs/vx_inode.h> */
+    int d1;
+};
+#undef	fs_bsize
+#define	uint16_t	VXFS_uint16_t
+
+#  if	defined(HASVXFS_OFF64_T)
+#define	off64_t		VXFS_off64_t
+#  endif	/* defined(HASVXFS_OFF64_T) */
+
+# if	defined(HASVXFS_SOLARIS_H) && !defined(HASVXFS_U64_T)
+#include <sys/fs/vx_solaris.h>
+# endif	/* defined(HASVXFS_SOLARIS_H) && !defined(HASVXFS_U64_T) */
+
+#include <sys/fs/vx_layout.h>
+#include <sys/fs/vx_const.h>
+#include <sys/fs/vx_mlink.h>
+# endif	/* defined(HASVXFS_SOLARIS_H) */
 
 #include <sys/fs/vx_inode.h>
 
@@ -60,14 +91,18 @@ static char *rcsid = "$Id: dnode2.c,v 1.5 97/10/24 07:55:47 abe Exp $";
  */
 
 int
-read_vxnode(va, v, vfs, dev, ino, sz)
-	caddr_t va;			/* containing vnode's address */
+read_vxnode(va, v, vfs, dev, ino, nl, sz, vnops)
+	KA_T va;			/* containing vnode's address */
 	struct vnode *v;		/* containing vnode */
 	struct l_vfs *vfs;		/* local vfs structure */
 	dev_t *dev;			/* device number receiver */
 	unsigned long *ino;		/* inode number receiver */
+	long *nl;			/* number of links receiver */
 	unsigned long *sz;		/* size receiver */
+	KA_T *vnops;			/* table of VxFS v_op values */
 {
+	struct vnode cv;
+	char tbuf[32];
 
 # if	defined(HASVXFS_VX_INODE)
 	struct vx_inode vx;
@@ -75,12 +110,35 @@ read_vxnode(va, v, vfs, dev, ino, sz)
 	struct inode vx;
 # endif	/* defined(HASVXFS_VX_INODE) */
 
+/*
+ * See if this is vnode is served by fdd_chain_vnops.  If it is, its
+ * v_data pointer leads to the "real" vnode.
+ */
+	if (v->v_data && v->v_op && (VXVOP_FDDCH < VXVOP_NUM)
+	&&  vnops[VXVOP_FDDCH] && ((KA_T)v->v_op == vnops[VXVOP_FDDCH]))
+	{
+	    if (kread((KA_T)v->v_data, (char *)&cv, sizeof(cv))) {
+		(void) sprintf(Namech,
+		    "node at %s: can't read real vx vnode: %s",
+		    print_kptr(va, tbuf),
+		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		return(1);
+	    }
+
+# if	defined(HASNCACHE)
+	    Lf->na = (KA_T)v->v_data;
+# endif	/* defined(HASNCACHE) */
+
+	    *v = cv;
+	    Ntype = vop2ty(v);
+	}
 /*
  * Read vnode's vx_inode.
  */
 	if (!v->v_data || kread((KA_T)v->v_data, (char *)&vx, sizeof(vx))) {
-	    (void) sprintf(Namech, "node at %#lx: can't read vx_inode: %#lx",
-		(unsigned long)va, (unsigned long)v->v_data);
+	    (void) sprintf(Namech, "node at %s: can't read vx_inode: %s",
+		print_kptr(va, tbuf),
+		print_kptr((KA_T)v->v_data, (char *)NULL));
 	    return(1);
 	}
 /*
@@ -94,6 +152,10 @@ read_vxnode(va, v, vfs, dev, ino, sz)
 	    *dev = (dev_t)vx.i_dev;
 	*ino = (unsigned long)vx.i_number;
 	*sz = (unsigned long)vx.i_size;
+/*
+ * Return link count.
+ */
+	*nl = (long)vx.i_nlink;
 	return(0);
 }
 #endif	/* defined(HASVXFS) */
