@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.4 98/03/06 08:38:36 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.17 2000/05/31 13:11:44 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -46,8 +46,61 @@ static char *rcsid = "$Id: dproc.c,v 1.4 98/03/06 08:38:36 abe Exp $";
  * Local definitions
  */
 
+#if	defined(solaris) && solaris>=20501
+#define	KVMHASHBN	8192		/* KVM hash bucket count -- MUST BE
+					 * A POWER OF 2!!! */
+#define	HASHKVM(va)	((int)((va * 31415) >> 3) & (KVMHASHBN-1))
+					/* virtual address hash function */
+
+# if	solaris<70000
+#define	KAERR	(u_longlong_t)-1	/* kvm_physaddr() error return */
+#define	KBUFT	char			/* kernel read buffer type */
+#define	KPHYS	u_longlong_t		/* kernel physical address type */
+#define	KVIRT	u_int			/* kernel virtual address type */
+# else	/* solaris>=70000 */
+#define	KAERR	(uint64_t)-1		/* kvm_physaddr() error return */
+#define	KBUFT	void			/* kernel read buffer type */
+#define	KPHYS	uint64_t		/* kernel physical address type */
+#define	KVIRT	uintptr_t		/* kernel virtual address type */
+# endif	/* solaris<70000 */
+#endif	/* defined(solaris) && solaris>=20501 */
+
+
+/*
+ * Local structures
+ */
+
+#if	defined(solaris) && solaris>=20501
+typedef struct kvmhash {
+	KVIRT vpa;			/* virtual page address */
+	KPHYS pa;			/* physical address */
+	struct kvmhash *nxt;		/* next virtual address */
+} kvmhash_t;
+#endif	/* defined(solaris) && solaris>=20501 */
+
+
+/*
+ * Local variables
+ */
+
+#if	defined(solaris) && solaris>=20501
+static struct as *Kas = (struct as *)NULL;
+					/* pointer to kernel's address space
+					 * map in kernel virtual memory */
+static kvmhash_t **KVMhb = (kvmhash_t **)NULL;
+					/* KVM hash buckets */
+static int PageSz = 0;			/* page size */
+static int PSMask = 0;			/* page size mask */
+static int PSShft = 0;			/* page size shift */
+
+# if	solaris<70000
+static struct as Kam;			/* kernel's address space map */
+static int Kmd = -1;			/* memory device file descriptor */
+#endif	/* solaris<70000 */
+#endif	/* defined(solaris) && solaris>=20501 */
+
 #if	defined(solaris) && solaris>=20500
-KA_T Kb = 0l;				/* KERNELBASE for Solaris 2.5 */
+static KA_T Kb = (KA_T)NULL;		/* KERNELBASE for Solaris 2.5 */
 #endif	/* defined(solaris) && solaris>=20500 */
 
 static int Knp = PROCDFLT;		/* numer of proc structures known
@@ -65,6 +118,10 @@ static int Switchgid = 0;		/* must switch GIDs for kvm_open() */
 #endif	/* defined(solaris) */
 
 
+/*
+ * Local function prototypes
+ */
+
 _PROTOTYPE(static void close_kvm,(void));
 _PROTOTYPE(static void get_kernel_access,(void));
 _PROTOTYPE(static void open_kvm,(void));
@@ -73,6 +130,14 @@ _PROTOTYPE(static void read_proc,(void));
 
 #if	defined(solaris)
 _PROTOTYPE(static void readfsinfo,(void));
+
+# if	solaris>=20501
+_PROTOTYPE(static void readkam,(KA_T addr));
+# endif	/* solaris>=20501 */
+
+#if	solaris>=20501 && solaris<70000
+_PROTOTYPE(extern u_longlong_t kvm_physaddr,(kvm_t *, struct as *, u_int));
+# endif	/* solaris>=20501 && solaris<70000 */
 #endif	/* defined(solaris) */
 
 
@@ -86,11 +151,21 @@ close_kvm()
 {
 	if (!Kd)
 	    return;
-	if (kvm_close(Kd) != 0) {
-	    (void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
-	    Exit(1);
+	if (Kd) {
+	    if (kvm_close(Kd) != 0) {
+		(void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
+		Exit(1);
+	    }
+	    Kd = (kvm_t *)NULL;
 	}
-	Kd = (kvm_t *)NULL;
+
+#if	defined(solaris) && solaris>=20501 && solaris<70000
+	if (Kmd >= 0) {
+	    (void) close(Kmd);
+	    Kmd = -1;
+	}
+#endif	/* defined(solaris) && solaris>=20501 && solaris<70000 */
+
 }
 
 
@@ -101,10 +176,12 @@ close_kvm()
 void
 gather_proc_info()
 {
-	int ft = 1;
+	MALLOC_S bc;
+	static int ft = 1;
 	int i, j;
 	struct proc *p;
 	int pgrp, pid, px;
+	long pofv;
 	short pss, sf;
 	struct user *u;
 	uid_t uid;
@@ -112,7 +189,17 @@ gather_proc_info()
 #if	defined(solaris)
 # if	solaris>=20400
 	int k;
-	uf_entry_t uf[NFPCHUNK];
+
+#  if	!defined(NFPCHUNK)
+#define	uf_ofile	uf_file
+#define	uf_pofile	uf_flag
+#define	u_flist		u_finfo.fi_list
+#define	u_nofiles	u_finfo.fi_nfiles
+#define	NFPREAD		64
+#  else	/* defined(NFPCHUNK) */
+#define	NFPREAD		NFPCHUNK
+#  endif	/* !defined(NFPCHUNK) */
+	uf_entry_t uf[NFPREAD];
 # endif	/* solaris>=20400 */
 # if	solaris>=20500
 	struct cred pc;
@@ -121,11 +208,16 @@ gather_proc_info()
 	struct ucwd cd;
 	int dl, nf;
 	struct file **uf;
-	unsigned ui;
 	static int xnf = 0;
 	struct file **xuf = (struct file **)NULL;
 #endif	/* solaris */
 
+#if	defined(HASFSTRUCT)
+	static int npofb = 0;
+	char *pofp;
+	static char *pofb = (char *)NULL;
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Do first-time only operations.
  */
@@ -134,6 +226,16 @@ gather_proc_info()
 		Sgvops = (KA_T)NULL;
 	    ft = 0;
 	}
+
+#if	defined(solaris) && solaris>=20501
+	else
+
+	/*
+	 * If not the first time, re-read the kernel's address space map.
+	 */
+	    readkam((KA_T)NULL);
+#endif	/* defined(solaris) && solaris>=20501 */
+
 /*
  * Read kernel name cache and process table.
  */
@@ -170,15 +272,14 @@ gather_proc_info()
 	/*
 	 * Read credentials for Solaris 2.5 and above process.
 	 */
-	    if (!p->p_cred
-	    ||  kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
+	    if (kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
 		continue;
-	    uid = pc.cr_ruid;
+	    uid = pc.cr_uid;
 # endif	/* solaris<20500 */
 #else	/* !defined(solaris) */
 	    pgrp = (int)p->p_pgrp;
 	    pid = p->p_pid;
-	    uid = p->p_uid;
+	    uid = p->p_suid ? p->p_suid : p->p_uid;
 #endif	/* defined(solaris) */
 
 	/*
@@ -283,7 +384,7 @@ gather_proc_info()
 #if	defined(solaris)
 	/*
 	 * Under Solaris the file pointers are stored in dynamically-linked
-	 * ufchunk structures, each containing NFPCHUNK file pointers.  The
+	 * ufchunk structures, each containing NFPREAD file pointers.  The
 	 * first ufchunk structure is in the user area.
 	 *
 	 * Under Solaris 2.4 the file pointers are in a dynamically allocated,
@@ -301,11 +402,11 @@ gather_proc_info()
 		}
 		if (!u->u_flist.uf_ofile[j-1])
 # else	/* solaris>=20400 */
-	    for (i = 0, j = NFPCHUNK; i < u->u_nofiles; i++) {
-		if (++j > NFPCHUNK) {
+	    for (i = 0, j = NFPREAD; i < u->u_nofiles; i++) {
+		if (++j > NFPREAD) {
 		    k = u->u_nofiles - i;
-		    if (k > NFPCHUNK)
-			k = NFPCHUNK;
+		    if (k > NFPREAD)
+			k = NFPREAD;
 		    if (kread((KA_T)((unsigned long)u->u_flist +
 				     i * sizeof(uf_entry_t)),
 				     (char*)&uf, k * sizeof(uf_entry_t)))
@@ -328,48 +429,81 @@ gather_proc_info()
 		+ (KA_T)p->p_uarea))
 	    {
 		nf = NOFILE_IN_U;
+		pofp = &u->u_pofile_arr[0];
 		uf = &u->u_ofile_arr[0];
 	    } else {
 		nf = u->u_lastfile + 1;
-		ui = nf * sizeof(struct file *);
-		if (!xuf) {
-		    xnf = nf;
-		    if (!(xuf = (struct file **)malloc(ui))) {
-			(void) fprintf(stderr, "%s: PID %d, malloc file[]\n",
-			    Pn, Lp->pid);
-			Exit(1);
-		    }
-		} else if (nf > xnf) {
-		    xnf = nf;
-		    if (!(xuf = (struct file **)realloc((MALLOC_P *)xuf, ui))) {
-			(void) fprintf(stderr, "%s: PID %d, realloc file[]\n",
-			     Pn, Lp->pid);
+		bc = (MALLOC_S)(nf * sizeof(struct file *));
+		if (bc >xnf) {
+		    if (!xuf)
+			xuf = (struct file **)malloc(bc);
+		    else
+			xuf = (struct file **)realloc((MALLOC_P *)xuf, bc);
+		    if (!xuf) {
+			(void) fprintf(stderr,
+			    "%s: PID %d: no file * space\n", Pn, Lp->pid);
 			Exit(1);
 		    }
+		    xnf = bc;
 		}
-		if (kread((KA_T)u->u_ofile, (char *)xuf, (int)ui))
+		if (kread((KA_T)u->u_ofile, (char *)xuf, (int)bc))
 		    nf = 0;
 		uf = xuf;
+
+# if	defined(HASFSTRUCT)
+		if (Fsv & FSV_FG) {
+		    bc = (MALLOC_S)nf;
+		    if (bc > npofb) {
+			if (!pofb)
+			    pofb = (char *)malloc(bc);
+			else
+			    pofb = (char *)realloc((MALLOC_P *)pofb, bc);
+			if (!pofb) {
+			    (void) fprintf(stderr,
+				"%s: PID %d: no pofile space\n", Pn, Lp->pid);
+			    Exit(1);
+			}
+			npofb = bc;
+		    }
+		    if (!u->u_pofile
+		    ||  kread((KA_T)u->u_pofile, pofb, bc))
+			zeromem(pofb, bc);
+		    pofp = pofb;
+		} else
+		    pofp = (char *)NULL;
+# endif	/* defined(HASFSTRUCT) */
+
 	    }
 	    *Cwd = *Rtd = '\0';
 	    for (i = 0; i < nf; i++) {
 		if (!uf[i])
 #endif	/* solaris */
+
 		    continue;
 		alloc_lfile((char *)NULL, i);
 
 #if	defined(solaris)
 # if	solaris<20400
+		pofv = (long)u->u_flist.uf_pofile[j-1];
 		process_file((KA_T)u->u_flist.uf_ofile[j-1]);
 # else	/* solaris>=20400 */
+		pofv = uf[j-1].uf_pofile;
 		process_file((KA_T)uf[j-1].uf_ofile);
 # endif	/* solaris <20400 */
 #else	/* !solaris */
+		pofv = pofp ? pofp[i] : 0l;
 		process_file((KA_T)uf[i]);
 #endif	/* solaris */
 
-		if (Lf->sf)
+		if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+		    if (Fsv & FSV_FG)
+			Lf->pof = pofv;
+#endif	/* defined(HASFSTRUCT) */
+
 		    link_lfile();
+		}
 	    }
 	/*
 	 * Examine results.
@@ -387,12 +521,68 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+	char er[64];
+	int i;
 	KA_T v;
 
 #if	defined(HAS_AFS)
 	struct nlist *nl = (struct nlist *)NULL;
 #endif	/* defined(HAS_AFS) */
 
+/*
+ * Check the Solaris or SunOS version number; check the SunOS architecture.
+ */
+	(void) ckkv(
+
+#if	defined(solaris)
+		    "Solaris", LSOF_VSTR, (char *)NULL, (char *)NULL
+#else	/* !defined(solaris) */
+		    "SunOS", LSOF_VSTR, (char *)NULL, LSOF_ARCH
+#endif	/* defined(solaris) */
+
+	);
+
+#if	defined(solaris) && solaris>=70000
+/*
+ * Compare the Solaris 7 lsof compilation bit size with the kernel bit size.
+ * Quit on a mismatch.
+ */
+	{
+	    char *cp, isa[1024];
+	    short kbits = 32;
+
+# if	defined(_LP64)
+	    short xkbits = 64;
+# else	/* !defined(_LP64) */
+	    short xkbits = 32;
+# endif	/* defined(_LP64) */
+
+	    if (sysinfo(SI_ISALIST, isa, (long)sizeof(isa)) < 0) {
+		(void) fprintf(stderr, "%s: can't get ISA list: %s\n",
+		    Pn, strerror(errno));
+		Exit(1);
+	    }
+	    for (cp = isa; *cp;) {
+		if (strncmp(cp, "sparcv9", strlen("sparcv9")) == 0) {
+		    kbits = 64;
+		    break;
+		}
+		if (!(cp = strchr(cp, ' ')))
+		    break;
+		cp++;
+	    }
+	    if (kbits != xkbits) {
+		(void) fprintf(stderr,
+		    "%s: FATAL: lsof was compiled for a %d bit kernel,\n",
+		    Pn, (int)xkbits);
+		(void) fprintf(stderr,
+		    "      but this machine has booted a %d bit kernel.\n",
+		    (int)kbits);
+		Exit(1);
+	    }
+	}
+#endif	/* defined(solaris) && solaris>=70000 */
+
 /*
  * Get kernel symbols.
  */
@@ -500,7 +690,7 @@ get_kernel_access()
 		Pn, print_kptr(v, (char *)NULL));
 	    Exit(1);
 	}
-# endif	/* solaris >=20500 */
+# endif	/* solaris>=20500 */
 
 /*
  * Get the Solaris clone major device number, if possible.
@@ -509,6 +699,23 @@ get_kernel_access()
 	if (get_Nl_value("clmaj", Drive_Nl, &v) >= 0 && v
 	&&  kread((KA_T)v, (char *)&CloneMaj, sizeof(CloneMaj)) == 0)
 	    HaveCloneMaj = 1;
+#endif	/* defined(solaris) */
+
+#if	defined(solaris) && solaris>=20501
+/*
+ * Get the kernel's virtual to physical map structure for Solaris 2.5.1 and
+ * above.
+ */
+	if (get_Nl_value("kasp", Drive_Nl, &v) >= 0 && v) {
+	    PageSz = getpagesize();
+	    PSMask = PageSz - 1;
+	    for (i = 1, PSShft = 0; i < PageSz; i <<= 1, PSShft++)
+		;
+	    (void) readkam(v);
+	}
+#endif	/* defined(solaris) && solaris>=20501 */
+
+#if	defined(solaris)
 /*
  * The Solaris process table is a linked list whose head pointer is acquired
  * by open_kvm()'s call to kvm_open().  We call close_kvm() here so that the
@@ -576,25 +783,115 @@ kread(addr, buf, len)
 	char *buf;			/* buffer to receive data */
 	READLEN_T len;			/* length to read */
 {
-	int br;
+	register int br;
 /*
  * Because lsof reads kernel data and follows pointers found there at a
  * rate considerably slower than the kernel, lsof sometimes acquires
- * invalid pointers.  If the invalid pointers are fed to kvm_read(),
+ * invalid pointers.  If the invalid pointers are fed to kvm_[k]read(),
  * a segmentation violation may result, so legal kernel addresses are
  * limited by the value of the KERNELBASE symbol (Kb value from the
  * kernel's _kernelbase variable for Solaris 2.5 and above).
  */
 
 #if	defined(solaris) && solaris>=20500
+#define	KVMREAD	kvm_kread
 	if (addr < Kb)
 #else	/* !defined(solaris) || solaris<20500 */
+#define	KVMREAD	kvm_read
 	if (addr < (KA_T)KERNELBASE)
 #endif	/* defined(solaris) && solaris>=20500 */
 
-		return(1);
-	br = kvm_read(Kd, (u_long)addr, buf, len);
-	return((br == len) ? 0 : 1);
+	    return(1);
+
+#if	defined(solaris) && solaris>=20501
+
+/*
+ * Make sure the virtual address represents real physical memory by testing
+ * it with kvm_physaddr().
+ *
+ * For Solaris below 7 read the kernel data with llseek() and read().  For
+ * Solaris 7 and above use kvm_pread().
+ */
+	if (Kas) {
+
+# if	solaris>20501
+	    register int b2r;
+	    register char *bp;
+# endif	/* solaris>20501 */
+
+	    register int h, ip, tb;
+	    register kvmhash_t *kp;
+	    KPHYS pa;
+	    register KVIRT va, vpa;
+
+# if	solaris<20600
+	    for (tb = 0, va = (KVIRT)addr;
+		 tb < len;
+		 tb += br, va += (KVIRT)br)
+# else	/* solaris>=20600 */
+	    for (bp = buf, tb = 0, va = (KVIRT)addr;
+		 tb < len;
+		 bp += br, tb += br, va += (KVIRT)br)
+# endif	/* solaris<20600 */
+
+	    {
+		vpa = (va & (KVIRT)~PSMask) >> PSShft;
+		ip = (int)(va & (KVIRT)PSMask);
+		h = HASHKVM(vpa);
+		for (kp = KVMhb[h]; kp; kp = kp->nxt) {
+		    if (kp->vpa == vpa) {
+			pa = kp->pa;
+			break;
+		    }
+		}
+		if (!kp) {
+		    if ((pa = kvm_physaddr(Kd, Kas, va)) == KAERR)
+			return(1);
+		    if (!(kp = (kvmhash_t *)malloc(sizeof(kvmhash_t)))) {
+			(void) fprintf(stderr, "%s: no kvmhash_t space\n", Pn);
+			Exit(1);
+		    }
+		    kp->nxt = KVMhb[h];
+		    pa = kp->pa = (pa & ~(KPHYS)PSMask);
+		    kp->vpa = vpa;
+		    KVMhb[h] = kp;
+		}
+
+# if	solaris<20600
+		br = (int)(len - tb);
+		if ((ip + br) > PageSz)
+		    br = PageSz - ip;
+# else	/* solaris>=20600 */
+		b2r = (int)(len - tb);
+		if ((ip + b2r) > PageSz)
+		    b2r = PageSz - ip;
+		pa |= (KPHYS)ip;
+
+#  if	solaris<70000
+		if (llseek(Kmd, (offset_t)pa, SEEK_SET) == (offset_t)-1)
+		    return(1);
+		if ((br = (int)read(Kmd, (void *)bp, (size_t)b2r)) <= 0)
+		    return(1);
+#  else	/* solaris>=70000 */
+		if ((br = kvm_pread(Kd, pa, (void *)bp, (size_t)b2r)) <= 0)
+		    return(1);
+#  endif	/* solaris<70000 */
+# endif	/* solaris<20600 */
+
+	    }
+
+# if	solaris>=20600
+	    return(0);
+# endif	/* solaris>=20600 */
+
+	}
+#endif	/* defined(solaris) && solaris>=20501 */
+
+/*
+ * Use kvm_read for Solaris < 2.5; use kvm_kread() Solaris >= 2.5.
+ */
+	br = KVMREAD(Kd, (u_long)addr, buf, len);
+	return(((READLEN_T)br == len) ? 0 : 1);
 }
 
 
@@ -633,6 +930,14 @@ open_kvm()
 	    Exit(1);
 	}
 
+#if	defined(solaris) && solaris>=20501 && solaris<70000
+	if ((Kmd = open((Memory ? Memory : KMEM), O_RDONLY)) < 0) {
+	    (void) fprintf(stderr, "%s: open(\"/dev/mem\"): %s\n", Pn, 
+		strerror(errno));
+	    Exit(1);
+	}
+#endif	/* defined(solaris) && solaris>=20501 && solaris<70000 */
+
 #if	defined(WILLDROPGID)
 /*
  * If this process has setgid permission, and is willing to surrender it,
@@ -670,11 +975,7 @@ process_text(pa)
 	struct as as;
 	int i, j, k, l;
 	struct seg s;
-	union segdata {
-	    struct segvn_data vn;
-	    struct segdev_data dev;
-	    struct segmap_data map;
-	} sd;
+	struct segvn_data vn;
 	KA_T v[MAXSEGS];
 /*
  * Get address space description.
@@ -699,9 +1000,9 @@ process_text(pa)
 	    ||  kread((KA_T)s.S_NEXT, (char *)&s, sizeof(s)))
 		break;
 	    if ((KA_T)s.s_ops == Sgvops && s.s_data) {
-		if (kread((KA_T)s.s_data, (char *)&sd, sizeof(sd)))
+		if (kread((KA_T)s.s_data, (char *)&vn, sizeof(vn)))
 		    break;
-		if (sd.vn.vp) {
+		if (vn.vp) {
 			
 		/*
 		 * This is a virtual node segment.
@@ -710,7 +1011,7 @@ process_text(pa)
 		 * print its information.
 		 */
 		    for (l = 0; l < k; l++) {
-			if (v[l] == (KA_T)sd.vn.vp)
+			if (v[l] == (KA_T)vn.vp)
 			    break;
 		    }
 		    if (l >= k) {
@@ -720,12 +1021,12 @@ process_text(pa)
 			FILEPTR = (struct file *)NULL;
 #endif	/* defined(FILEPTR) */
 
-			process_node((KA_T)sd.vn.vp);
+			process_node((KA_T)vn.vp);
 			if (Lf->sf) {
 			    link_lfile();
 			    i++;
 			}
-			v[k++] = (KA_T)sd.vn.vp;
+			v[k++] = (KA_T)vn.vp;
 		    }
 		}
 	    }
@@ -794,6 +1095,58 @@ readfsinfo()
 
 	}
 }
+
+
+# if	solaris>=20501
+/*
+ * readkam() - read kernel's address map structure
+ */
+
+static void
+readkam(addr)
+	KA_T addr;			/* kernel virtual address */
+{
+	register int i;
+	register kvmhash_t *kp, *kpp;
+	static KA_T kas = (KA_T)NULL;
+	size_t sz;
+
+	if (addr)
+	    kas = addr;
+	Kas = (struct as *)NULL;
+
+# if	solaris<70000
+	if (kas && !kread(kas, (char *)&Kam, sizeof(Kam)))
+	    Kas = (KA_T)&Kam;
+# else	/* solaris>=70000 */
+	Kas = (struct as *)kas;
+# endif	/* solaris<70000 */
+
+	if (Kas) {
+	    if (!KVMhb) {
+		if (!(KVMhb = (kvmhash_t **)calloc(KVMHASHBN,
+						   sizeof(kvmhash_t *))))
+		{
+		     (void) fprintf(stderr,
+			"%s: no space (%d) for KVM hash buckets\n",
+			(KVMHASHBN * sizeof(kvmhash_t *)), Pn);
+		    Exit(1);
+		}
+	    } else if (!addr) {
+		for (i = 0; i < KVMHASHBN; i++) {
+		    if ((kp = KVMhb[i])) {
+			while (kp) {
+			    kpp = kp->nxt;
+			    (void) free((void *)kp);
+			    kp = kpp;
+			}
+			KVMhb[i] = (kvmhash_t *)NULL;
+		    }
+		}
+	    }
+	}
+}
+# endif	/* solaris>=20501 */
 #endif	/* solaris */
 
 
@@ -867,6 +1220,14 @@ read_proc()
 		    continue;
 
 #if	defined(solaris)
+# if	solaris >=20500
+		/*
+		 * Check Solaris 2.5 and above p_cred pointer.
+		 */
+	    	    if (!p->p_cred)
+			continue;
+# endif	/* solaris >=20500 */
+
 		/*
 		 * Read Solaris PGRP and PID numbers.
 		 */
