@@ -33,13 +33,14 @@
 
 
 /*
- * $Id: dproto.h,v 1.4 98/03/06 08:38:38 abe Exp $
+ * $Id: dproto.h,v 1.9 2000/06/25 17:31:11 abe Exp $
  */
 
 
 _PROTOTYPE(extern void completevfs,(struct l_vfs *vfs, dev_t *dev));
 _PROTOTYPE(extern int is_file_named,(char *p, int nt, enum vtype vt, int ps));
 _PROTOTYPE(extern struct l_vfs *readvfs,(KA_T ka, struct vfs *la, struct vnode *lv));
+_PROTOTYPE(extern int vop2ty,(struct vnode *vp));
 
 #if	defined(HAS_AFS)
 _PROTOTYPE(extern struct vnode *alloc_vcache,(void));
@@ -57,11 +58,12 @@ _PROTOTYPE(extern int rw_pseudo_sect,(int m));
 #endif	/* defined(HASDCACHE) */
 
 #if	defined(HASVXFS)
-_PROTOTYPE(extern int read_vxnode,(KA_T va, struct vnode *v, struct l_vfs *vfs,dev_t *dev, unsigned long *ino, unsigned long *sz));
+_PROTOTYPE(extern int read_vxnode,(KA_T va, struct vnode *v, struct l_vfs *vfs,dev_t *dev, unsigned long *ino, long *nl, unsigned long *sz, KA_T *vnops));
 #endif	/* defined(HASVXFS) */
 
 #if	defined(solaris)
 _PROTOTYPE(extern void process_socket,(KA_T sa, char *ty));
+_PROTOTYPE(extern void read_clone,(void));
 #else	/* !defined(solaris) */
 _PROTOTYPE(extern void process_socket,(KA_T sa));
 #endif	/* defined(solaris) */
