@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rnch.c,v 1.7 98/11/25 06:34:44 abe Exp $";
+static char *rcsid = "$Id: rnch.c,v 1.8 2000/05/31 12:59:41 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -89,6 +89,11 @@ static char *rcsid = "$Id: rnch.c,v 1.7 98/11/25 06:34:44 abe Exp $";
  *
  *				Deafult: namlen
  *
+ *	Define NCACHE_NEGVN	as the name of the name list element
+ *				whose value is a vnode address to
+ *				ignore when loading the kernel name
+ *				cache.
+ *
  *	Define NCACHE_NODEID	as the name of the element in the
  *				ncache structure that contains the
  *				vnode's capability ID.
@@ -147,6 +152,11 @@ static struct l_nch **Nchash = (struct l_nch **)NULL;
 					/* Ncache hash pointers */
 static int Ncfirst = 1;			/* first-call status */
 
+# if 	defined(NCACHE_NEGVN)
+static KA_T NegVN = (KA_T)NULL;		/* negative vnode address */
+static int NegVNSt = 0;			/* NegVN status: 0 = not loaded */
+# endif	/* defined(NCACHE_NEGVN) */
+
 # if	defined(NCACHE_NODEID)
 _PROTOTYPE(static struct l_nch *ncache_addr,(unsigned long i, KA_T v));
 #define ncachehash(i,v)		Nchash+(((((int)(v)>>2)+((int)(i)))*31415)&Mch)
@@ -368,6 +378,19 @@ ncache_load()
 		}
 		iNc = Nc = DEFNCACHESZ;
 	    }
+
+# if	defined(NCACHE_NEGVN)
+	/*
+	 * Get negative vnode address.
+	 */
+	    if (!NegVNSt) {
+		if (get_Nl_value(NCACHE_NEGVN, (struct drive_Nl *)NULL, &NegVN)
+		< 0)
+		    NegVN = (KA_T)NULL;
+		NegVNSt = 1;
+	    }
+# endif	/* defined(NCACHE_NEGVN) */
+
 	/*
 	 * Establish kernel cache address.
 	 */
@@ -491,6 +514,11 @@ no_local_space:
 	    if (!kc->NCACHE_VP || (len = kc->NCACHE_NAMLEN) < 1)
 		continue;
 
+# if	defined(NCACHE_NEGVN)
+	    if (NegVN && ((KA_T)kc->NCACHE_VP == NegVN))
+		continue;
+# endif	/* defined(NCACHE_NEGVN) */
+
 # if	defined(HASDNLCPTR)
 	/*
 	 * Read name from kernel to a temporary buffer.
@@ -671,6 +699,13 @@ no_local_space:
 	    if (!lc->dp)
 		continue;
 
+# if	defined(NCACHE_NEGVN)
+	     if (NegVN && (lc->dp == NegVN)) {
+		lc->pa = (struct l_nch *)NULL;
+		continue;
+	     }
+# endif	/* defined(NCACHE_NEGVN) */
+
 # if	defined(NCACHE_NODEID)
 	    lc->pa = ncache_addr(lc->did, lc->dp);
 # else	/* !defined(NCACHE_NODEID) */
