@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: proto.h,v 1.9 98/03/06 07:31:44 abe Exp $
+ * $Id: proto.h,v 1.20 2000/03/16 15:25:18 abe Exp $
  */
 
 
@@ -66,7 +66,8 @@
 _PROTOTYPE(extern void alloc_lfile,(char *nm, int num));
 _PROTOTYPE(extern void alloc_lproc,(int pid, int pgrp, int ppid, UID_ARG uid, char *cmd, int pss, int sf));
 _PROTOTYPE(extern void childx,(void));
-_PROTOTYPE(extern int ck_file_arg,(int i, int ac, char *av[]));
+_PROTOTYPE(extern int ck_file_arg,(int i, int ac, char *av[], int fv, int rs, struct stat *sbp));
+_PROTOTYPE(extern void ckkv,(char *d, char *er, char *ev, char *ea));
 _PROTOTYPE(extern void clr_devtab,(void));
 _PROTOTYPE(extern int compdev,(COMP_P *a1, COMP_P *a2));
 _PROTOTYPE(extern int comppid,(COMP_P *a1, COMP_P *a2));
@@ -77,6 +78,7 @@ _PROTOTYPE(extern void dropgid,(void));
 
 _PROTOTYPE(extern char *endnm,(void));
 _PROTOTYPE(extern void enter_dev_ch,(char *m));
+_PROTOTYPE(extern int enter_dir,(char *d, int descend));
 _PROTOTYPE(extern int enter_fd,(char *f));
 _PROTOTYPE(extern int enter_network_address,(char *na));
 _PROTOTYPE(extern int enter_id,(enum IDType ty, char *p));
@@ -84,45 +86,49 @@ _PROTOTYPE(extern void enter_nm,(char *m));
 _PROTOTYPE(extern int enter_str_lst,(char *opt, char *s, struct str_lst **lp));
 _PROTOTYPE(extern int enter_uid,(char *u));
 _PROTOTYPE(extern int examine_lproc,(void));
-_PROTOTYPE(extern void ent_inaddr,(struct in_addr *la, int lp, struct in_addr *fa, int fp));
+_PROTOTYPE(extern void ent_inaddr,(unsigned char *la, int lp, unsigned char *fa, int fp, int af));
 _PROTOTYPE(extern void Exit,(int xv)) exiting;
 _PROTOTYPE(extern void find_ch_ino,(void));
 _PROTOTYPE(extern void free_lproc,(struct lproc *lp));
 _PROTOTYPE(extern void gather_proc_info,(void));
-_PROTOTYPE(extern char *gethostnm,(struct in_addr *ia));
+_PROTOTYPE(extern char *gethostnm,(unsigned char *ia, int af));
+_PROTOTYPE(extern int hashbyname,(char *nm, int mod));
+_PROTOTYPE(extern void hashSfile,(void));
 _PROTOTYPE(extern void initialize,(void));
 _PROTOTYPE(extern int is_cmd_excl,(char *cmd, short *pss, short *sf));
 _PROTOTYPE(extern int is_file_sel,(struct lfile *lf));
-_PROTOTYPE(extern int is_nw_addr,(struct in_addr *ia, int p));
+_PROTOTYPE(extern int is_nw_addr,(unsigned char *ia, int p, int af));
 _PROTOTYPE(extern int is_proc_excl,(int pid, int pgrp, UID_ARG uid, short *pss, short *sf));
 _PROTOTYPE(extern int is_readable,(char *path, int msg));
 _PROTOTYPE(extern int kread,(KA_T addr, char *buf, READLEN_T len));
 _PROTOTYPE(extern void link_lfile,(void));
 _PROTOTYPE(extern struct l_dev *lkupdev,(dev_t *dev, int i, int r));
 _PROTOTYPE(extern int main,(int argc, char *argv[]));
+_PROTOTYPE(extern int lstatsafely,(char *path, struct stat *buf));
 _PROTOTYPE(extern char *mkstrcpy,(char *src, MALLOC_S *rlp));
 _PROTOTYPE(extern char *mkstrcat,(char *s1, int l1, char *s2, int l2, char *s3, int l3, MALLOC_S *clp));
 _PROTOTYPE(extern int printchdevname,(dev_t *rdev, int f));
 _PROTOTYPE(extern void print_file,(void));
 _PROTOTYPE(extern void print_init,(void));
-_PROTOTYPE(extern void printiproto,(int p));
 _PROTOTYPE(extern void printname,(int nl));
 _PROTOTYPE(extern char *print_kptr,(KA_T kp, char *buf));
-_PROTOTYPE(extern void print_proc,(void));
+_PROTOTYPE(extern int print_proc,(void));
 _PROTOTYPE(extern void printrawaddr,(struct sockaddr *sa));
 _PROTOTYPE(extern void print_tcptpi,(int nl));
 _PROTOTYPE(extern char *printuid,(UID_ARG u, int *ty));
-_PROTOTYPE(extern void printunkaf,(int fam));
+_PROTOTYPE(extern void printunkaf,(int fam, int ty));
+_PROTOTYPE(extern char *printsockty,(int ty));
 _PROTOTYPE(extern void process_file,(KA_T fp));
 _PROTOTYPE(extern void process_node,(KA_T f));
 _PROTOTYPE(extern char *Readlink,(char *arg));
 _PROTOTYPE(extern void readdev,(int skip));
-_PROTOTYPE(extern int readmnt,(void));
+_PROTOTYPE(extern struct mounts *readmnt,(void));
 _PROTOTYPE(extern void rereaddev,(void));
-_PROTOTYPE(extern int safestrlen,(char *sp));
-_PROTOTYPE(extern void safestrprtn,(char *sp, int len, FILE *fs, int nl));
-_PROTOTYPE(extern void safestrprt,(char *sp, FILE *fs, int nl));
+_PROTOTYPE(extern int safestrlen,(char *sp, int flags));
+_PROTOTYPE(extern void safestrprtn,(char *sp, int len, FILE *fs, int flags));
+_PROTOTYPE(extern void safestrprt,(char *sp, FILE *fs, int flags));
 _PROTOTYPE(extern int statsafely,(char *path, struct stat *buf));
+_PROTOTYPE(extern void stkdir,(char *p));
 _PROTOTYPE(extern void usage,(int xv, int fh, int version));
 _PROTOTYPE(extern int vfy_dev,(struct l_dev *dp));
 _PROTOTYPE(extern char *x2dev,(char *s, dev_t *d));
@@ -153,6 +159,10 @@ _PROTOTYPE(extern void write_dcache,(void));
 _PROTOTYPE(extern int readfifonode,(KA_T fa, struct fifonode *f));
 #endif	/* defined(HASFIFONODE) */
 
+#if	defined(HASFSTRUCT)
+_PROTOTYPE(extern char *print_fflags,(long ffg, long pof));
+#endif	/* defined(HASFSTRUCT) */
+
 #if	defined(HASGNODE)
 _PROTOTYPE(extern int readgnode,(KA_T ga, struct gnode *g));
 #endif	/* defined(HASGNODE) */
@@ -191,10 +201,6 @@ _PROTOTYPE(extern char *HASPRINTINO,(struct lfile *lf));
 _PROTOTYPE(extern void HASPRINTNM,(struct lfile *lf));
 #endif	/* defined(HASPRINTNM) */
 
-#if	defined(HASPRIVNMCACHE)
-_PROTOTYPE(extern int HASPRIVNMCACHE,(struct lfile *lf));
-#endif	/* defined(HASPRIVNMCACHE) */
-
 #if	defined(HASPRINTOFF)
 _PROTOTYPE(extern char *HASPRINTOFF,(struct lfile *lf, int ty));
 #endif	/* defined(HASPRINTOFF) */
@@ -203,6 +209,14 @@ _PROTOTYPE(extern char *HASPRINTOFF,(struct lfile *lf, int ty));
 _PROTOTYPE(extern char *HASPRINTSZ,(struct lfile *lf));
 #endif	/* defined(HASPRINTSZ) */
 
+#if	defined(HASPRIVNMCACHE)
+_PROTOTYPE(extern int HASPRIVNMCACHE,(struct lfile *lf));
+#endif	/* defined(HASPRIVNMCACHE) */
+
+#if	!defined(HASPRIVPRIPP)
+_PROTOTYPE(extern void printiproto,(int p));
+#endif	/* !defined(HASPRIVPRIPP) */
+
 #if	defined(HASRNODE)
 _PROTOTYPE(extern int readrnode,(KA_T ra, struct rnode *r));
 #endif	/* defined(HASRNODE) */
